from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MassFluxUnits(Enum):
        """
            MassFluxUnits enumeration
        """
        
        GramPerSecondPerSquareMeter = 'GramPerSecondPerSquareMeter'
        """
            
        """
        
        GramPerSecondPerSquareCentimeter = 'GramPerSecondPerSquareCentimeter'
        """
            
        """
        
        GramPerSecondPerSquareMillimeter = 'GramPerSecondPerSquareMillimeter'
        """
            
        """
        
        GramPerHourPerSquareMeter = 'GramPerHourPerSquareMeter'
        """
            
        """
        
        GramPerHourPerSquareCentimeter = 'GramPerHourPerSquareCentimeter'
        """
            
        """
        
        GramPerHourPerSquareMillimeter = 'GramPerHourPerSquareMillimeter'
        """
            
        """
        
        KilogramPerSecondPerSquareMeter = 'KilogramPerSecondPerSquareMeter'
        """
            
        """
        
        KilogramPerSecondPerSquareCentimeter = 'KilogramPerSecondPerSquareCentimeter'
        """
            
        """
        
        KilogramPerSecondPerSquareMillimeter = 'KilogramPerSecondPerSquareMillimeter'
        """
            
        """
        
        KilogramPerHourPerSquareMeter = 'KilogramPerHourPerSquareMeter'
        """
            
        """
        
        KilogramPerHourPerSquareCentimeter = 'KilogramPerHourPerSquareCentimeter'
        """
            
        """
        
        KilogramPerHourPerSquareMillimeter = 'KilogramPerHourPerSquareMillimeter'
        """
            
        """
        

class MassFluxDto:
    """
    A DTO representation of a MassFlux

    Attributes:
        value (float): The value of the MassFlux.
        unit (MassFluxUnits): The specific unit that the MassFlux value is representing.
    """

    def __init__(self, value: float, unit: MassFluxUnits):
        """
        Create a new DTO representation of a MassFlux

        Parameters:
            value (float): The value of the MassFlux.
            unit (MassFluxUnits): The specific unit that the MassFlux value is representing.
        """
        self.value: float = value
        """
        The value of the MassFlux
        """
        self.unit: MassFluxUnits = unit
        """
        The specific unit that the MassFlux value is representing
        """

    def to_json(self):
        """
        Get a MassFlux DTO JSON object representing the current unit.

        :return: JSON object represents MassFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KilogramPerSecondPerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MassFlux DTO from a json representation.

        :param data: The MassFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KilogramPerSecondPerSquareMeter"}
        :return: A new instance of MassFluxDto.
        :rtype: MassFluxDto
        """
        return MassFluxDto(value=data["value"], unit=MassFluxUnits(data["unit"]))


class MassFlux(AbstractMeasure):
    """
    Mass flux is the mass flow rate per unit area.

    Args:
        value (float): The value.
        from_unit (MassFluxUnits): The MassFlux unit to create from, The default unit is KilogramPerSecondPerSquareMeter
    """
    def __init__(self, value: float, from_unit: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__grams_per_second_per_square_meter = None
        
        self.__grams_per_second_per_square_centimeter = None
        
        self.__grams_per_second_per_square_millimeter = None
        
        self.__grams_per_hour_per_square_meter = None
        
        self.__grams_per_hour_per_square_centimeter = None
        
        self.__grams_per_hour_per_square_millimeter = None
        
        self.__kilograms_per_second_per_square_meter = None
        
        self.__kilograms_per_second_per_square_centimeter = None
        
        self.__kilograms_per_second_per_square_millimeter = None
        
        self.__kilograms_per_hour_per_square_meter = None
        
        self.__kilograms_per_hour_per_square_centimeter = None
        
        self.__kilograms_per_hour_per_square_millimeter = None
        

    def convert(self, unit: MassFluxUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter) -> MassFluxDto:
        """
        Get a new instance of MassFlux DTO representing the current unit.

        :param hold_in_unit: The specific MassFlux unit to store the MassFlux value in the DTO representation.
        :type hold_in_unit: MassFluxUnits
        :return: A new instance of MassFluxDto.
        :rtype: MassFluxDto
        """
        return MassFluxDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter):
        """
        Get a MassFlux DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MassFlux unit to store the MassFlux value in the DTO representation.
        :type hold_in_unit: MassFluxUnits
        :return: JSON object represents MassFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KilogramPerSecondPerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(mass_flux_dto: MassFluxDto):
        """
        Obtain a new instance of MassFlux from a DTO unit object.

        :param mass_flux_dto: The MassFlux DTO representation.
        :type mass_flux_dto: MassFluxDto
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(mass_flux_dto.value, mass_flux_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MassFlux from a DTO unit json representation.

        :param data: The MassFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KilogramPerSecondPerSquareMeter"}
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux.from_dto(MassFluxDto.from_json(data))

    def __convert_from_base(self, from_unit: MassFluxUnits) -> float:
        value = self._value
        
        if from_unit == MassFluxUnits.GramPerSecondPerSquareMeter:
            return (value * 1e3)
        
        if from_unit == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return (value * 1e-1)
        
        if from_unit == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return (value * 1e-3)
        
        if from_unit == MassFluxUnits.GramPerHourPerSquareMeter:
            return (value * 3.6e6)
        
        if from_unit == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return (value * 3.6e2)
        
        if from_unit == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return (value * 3.6e0)
        
        if from_unit == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return ((value * 1e3) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return ((value * 1e-1) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return ((value * 1e-3) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return ((value * 3.6e6) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return ((value * 3.6e2) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return ((value * 3.6e0) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MassFluxUnits) -> float:
        
        if to_unit == MassFluxUnits.GramPerSecondPerSquareMeter:
            return (value / 1e3)
        
        if to_unit == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return (value / 1e-1)
        
        if to_unit == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return (value / 1e-3)
        
        if to_unit == MassFluxUnits.GramPerHourPerSquareMeter:
            return (value / 3.6e6)
        
        if to_unit == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return (value / 3.6e2)
        
        if to_unit == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return (value / 3.6e0)
        
        if to_unit == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return ((value / 1e3) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return ((value / 1e-1) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return ((value / 1e-3) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return ((value / 3.6e6) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return ((value / 3.6e2) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return ((value / 3.6e0) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_grams_per_second_per_square_meter(grams_per_second_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_second_per_square_meter.

        

        :param meters: The MassFlux value in grams_per_second_per_square_meter.
        :type grams_per_second_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_second_per_square_meter, MassFluxUnits.GramPerSecondPerSquareMeter)

    
    @staticmethod
    def from_grams_per_second_per_square_centimeter(grams_per_second_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_second_per_square_centimeter.

        

        :param meters: The MassFlux value in grams_per_second_per_square_centimeter.
        :type grams_per_second_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_second_per_square_centimeter, MassFluxUnits.GramPerSecondPerSquareCentimeter)

    
    @staticmethod
    def from_grams_per_second_per_square_millimeter(grams_per_second_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_second_per_square_millimeter.

        

        :param meters: The MassFlux value in grams_per_second_per_square_millimeter.
        :type grams_per_second_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_second_per_square_millimeter, MassFluxUnits.GramPerSecondPerSquareMillimeter)

    
    @staticmethod
    def from_grams_per_hour_per_square_meter(grams_per_hour_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_hour_per_square_meter.

        

        :param meters: The MassFlux value in grams_per_hour_per_square_meter.
        :type grams_per_hour_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_hour_per_square_meter, MassFluxUnits.GramPerHourPerSquareMeter)

    
    @staticmethod
    def from_grams_per_hour_per_square_centimeter(grams_per_hour_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_hour_per_square_centimeter.

        

        :param meters: The MassFlux value in grams_per_hour_per_square_centimeter.
        :type grams_per_hour_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_hour_per_square_centimeter, MassFluxUnits.GramPerHourPerSquareCentimeter)

    
    @staticmethod
    def from_grams_per_hour_per_square_millimeter(grams_per_hour_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_hour_per_square_millimeter.

        

        :param meters: The MassFlux value in grams_per_hour_per_square_millimeter.
        :type grams_per_hour_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_hour_per_square_millimeter, MassFluxUnits.GramPerHourPerSquareMillimeter)

    
    @staticmethod
    def from_kilograms_per_second_per_square_meter(kilograms_per_second_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_second_per_square_meter.

        

        :param meters: The MassFlux value in kilograms_per_second_per_square_meter.
        :type kilograms_per_second_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_second_per_square_meter, MassFluxUnits.KilogramPerSecondPerSquareMeter)

    
    @staticmethod
    def from_kilograms_per_second_per_square_centimeter(kilograms_per_second_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_second_per_square_centimeter.

        

        :param meters: The MassFlux value in kilograms_per_second_per_square_centimeter.
        :type kilograms_per_second_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_second_per_square_centimeter, MassFluxUnits.KilogramPerSecondPerSquareCentimeter)

    
    @staticmethod
    def from_kilograms_per_second_per_square_millimeter(kilograms_per_second_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_second_per_square_millimeter.

        

        :param meters: The MassFlux value in kilograms_per_second_per_square_millimeter.
        :type kilograms_per_second_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_second_per_square_millimeter, MassFluxUnits.KilogramPerSecondPerSquareMillimeter)

    
    @staticmethod
    def from_kilograms_per_hour_per_square_meter(kilograms_per_hour_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_hour_per_square_meter.

        

        :param meters: The MassFlux value in kilograms_per_hour_per_square_meter.
        :type kilograms_per_hour_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_hour_per_square_meter, MassFluxUnits.KilogramPerHourPerSquareMeter)

    
    @staticmethod
    def from_kilograms_per_hour_per_square_centimeter(kilograms_per_hour_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_hour_per_square_centimeter.

        

        :param meters: The MassFlux value in kilograms_per_hour_per_square_centimeter.
        :type kilograms_per_hour_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_hour_per_square_centimeter, MassFluxUnits.KilogramPerHourPerSquareCentimeter)

    
    @staticmethod
    def from_kilograms_per_hour_per_square_millimeter(kilograms_per_hour_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_hour_per_square_millimeter.

        

        :param meters: The MassFlux value in kilograms_per_hour_per_square_millimeter.
        :type kilograms_per_hour_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_hour_per_square_millimeter, MassFluxUnits.KilogramPerHourPerSquareMillimeter)

    
    @property
    def grams_per_second_per_square_meter(self) -> float:
        """
        
        """
        if self.__grams_per_second_per_square_meter != None:
            return self.__grams_per_second_per_square_meter
        self.__grams_per_second_per_square_meter = self.__convert_from_base(MassFluxUnits.GramPerSecondPerSquareMeter)
        return self.__grams_per_second_per_square_meter

    
    @property
    def grams_per_second_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__grams_per_second_per_square_centimeter != None:
            return self.__grams_per_second_per_square_centimeter
        self.__grams_per_second_per_square_centimeter = self.__convert_from_base(MassFluxUnits.GramPerSecondPerSquareCentimeter)
        return self.__grams_per_second_per_square_centimeter

    
    @property
    def grams_per_second_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__grams_per_second_per_square_millimeter != None:
            return self.__grams_per_second_per_square_millimeter
        self.__grams_per_second_per_square_millimeter = self.__convert_from_base(MassFluxUnits.GramPerSecondPerSquareMillimeter)
        return self.__grams_per_second_per_square_millimeter

    
    @property
    def grams_per_hour_per_square_meter(self) -> float:
        """
        
        """
        if self.__grams_per_hour_per_square_meter != None:
            return self.__grams_per_hour_per_square_meter
        self.__grams_per_hour_per_square_meter = self.__convert_from_base(MassFluxUnits.GramPerHourPerSquareMeter)
        return self.__grams_per_hour_per_square_meter

    
    @property
    def grams_per_hour_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__grams_per_hour_per_square_centimeter != None:
            return self.__grams_per_hour_per_square_centimeter
        self.__grams_per_hour_per_square_centimeter = self.__convert_from_base(MassFluxUnits.GramPerHourPerSquareCentimeter)
        return self.__grams_per_hour_per_square_centimeter

    
    @property
    def grams_per_hour_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__grams_per_hour_per_square_millimeter != None:
            return self.__grams_per_hour_per_square_millimeter
        self.__grams_per_hour_per_square_millimeter = self.__convert_from_base(MassFluxUnits.GramPerHourPerSquareMillimeter)
        return self.__grams_per_hour_per_square_millimeter

    
    @property
    def kilograms_per_second_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilograms_per_second_per_square_meter != None:
            return self.__kilograms_per_second_per_square_meter
        self.__kilograms_per_second_per_square_meter = self.__convert_from_base(MassFluxUnits.KilogramPerSecondPerSquareMeter)
        return self.__kilograms_per_second_per_square_meter

    
    @property
    def kilograms_per_second_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_second_per_square_centimeter != None:
            return self.__kilograms_per_second_per_square_centimeter
        self.__kilograms_per_second_per_square_centimeter = self.__convert_from_base(MassFluxUnits.KilogramPerSecondPerSquareCentimeter)
        return self.__kilograms_per_second_per_square_centimeter

    
    @property
    def kilograms_per_second_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_second_per_square_millimeter != None:
            return self.__kilograms_per_second_per_square_millimeter
        self.__kilograms_per_second_per_square_millimeter = self.__convert_from_base(MassFluxUnits.KilogramPerSecondPerSquareMillimeter)
        return self.__kilograms_per_second_per_square_millimeter

    
    @property
    def kilograms_per_hour_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilograms_per_hour_per_square_meter != None:
            return self.__kilograms_per_hour_per_square_meter
        self.__kilograms_per_hour_per_square_meter = self.__convert_from_base(MassFluxUnits.KilogramPerHourPerSquareMeter)
        return self.__kilograms_per_hour_per_square_meter

    
    @property
    def kilograms_per_hour_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_hour_per_square_centimeter != None:
            return self.__kilograms_per_hour_per_square_centimeter
        self.__kilograms_per_hour_per_square_centimeter = self.__convert_from_base(MassFluxUnits.KilogramPerHourPerSquareCentimeter)
        return self.__kilograms_per_hour_per_square_centimeter

    
    @property
    def kilograms_per_hour_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_hour_per_square_millimeter != None:
            return self.__kilograms_per_hour_per_square_millimeter
        self.__kilograms_per_hour_per_square_millimeter = self.__convert_from_base(MassFluxUnits.KilogramPerHourPerSquareMillimeter)
        return self.__kilograms_per_hour_per_square_millimeter

    
    def to_string(self, unit: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the MassFlux to a string.
        
        Note: the default format for MassFlux is KilogramPerSecondPerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MassFlux. Default is 'KilogramPerSecondPerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MassFluxUnits.GramPerSecondPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.grams_per_second_per_square_meter, fractional_digits)} g·s⁻¹·m⁻²"""
        
        if unit == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.grams_per_second_per_square_centimeter, fractional_digits)} g·s⁻¹·cm⁻²"""
        
        if unit == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.grams_per_second_per_square_millimeter, fractional_digits)} g·s⁻¹·mm⁻²"""
        
        if unit == MassFluxUnits.GramPerHourPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.grams_per_hour_per_square_meter, fractional_digits)} g·h⁻¹·m⁻²"""
        
        if unit == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.grams_per_hour_per_square_centimeter, fractional_digits)} g·h⁻¹·cm⁻²"""
        
        if unit == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.grams_per_hour_per_square_millimeter, fractional_digits)} g·h⁻¹·mm⁻²"""
        
        if unit == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_second_per_square_meter, fractional_digits)} kg·s⁻¹·m⁻²"""
        
        if unit == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_second_per_square_centimeter, fractional_digits)} kg·s⁻¹·cm⁻²"""
        
        if unit == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_second_per_square_millimeter, fractional_digits)} kg·s⁻¹·mm⁻²"""
        
        if unit == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_hour_per_square_meter, fractional_digits)} kg·h⁻¹·m⁻²"""
        
        if unit == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_hour_per_square_centimeter, fractional_digits)} kg·h⁻¹·cm⁻²"""
        
        if unit == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_hour_per_square_millimeter, fractional_digits)} kg·h⁻¹·mm⁻²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter) -> str:
        """
        Get MassFlux unit abbreviation.
        Note! the default abbreviation for MassFlux is KilogramPerSecondPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MassFluxUnits.GramPerSecondPerSquareMeter:
            return """g·s⁻¹·m⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return """g·s⁻¹·cm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return """g·s⁻¹·mm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerHourPerSquareMeter:
            return """g·h⁻¹·m⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return """g·h⁻¹·cm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return """g·h⁻¹·mm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return """kg·s⁻¹·m⁻²"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return """kg·s⁻¹·cm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return """kg·s⁻¹·mm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return """kg·h⁻¹·m⁻²"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return """kg·h⁻¹·cm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return """kg·h⁻¹·mm⁻²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class AreaMomentOfInertiaUnits(Enum):
        """
            AreaMomentOfInertiaUnits enumeration
        """
        
        MeterToTheFourth = 'MeterToTheFourth'
        """
            
        """
        
        DecimeterToTheFourth = 'DecimeterToTheFourth'
        """
            
        """
        
        CentimeterToTheFourth = 'CentimeterToTheFourth'
        """
            
        """
        
        MillimeterToTheFourth = 'MillimeterToTheFourth'
        """
            
        """
        
        FootToTheFourth = 'FootToTheFourth'
        """
            
        """
        
        InchToTheFourth = 'InchToTheFourth'
        """
            
        """
        

class AreaMomentOfInertiaDto:
    """
    A DTO representation of a AreaMomentOfInertia

    Attributes:
        value (float): The value of the AreaMomentOfInertia.
        unit (AreaMomentOfInertiaUnits): The specific unit that the AreaMomentOfInertia value is representing.
    """

    def __init__(self, value: float, unit: AreaMomentOfInertiaUnits):
        """
        Create a new DTO representation of a AreaMomentOfInertia

        Parameters:
            value (float): The value of the AreaMomentOfInertia.
            unit (AreaMomentOfInertiaUnits): The specific unit that the AreaMomentOfInertia value is representing.
        """
        self.value: float = value
        """
        The value of the AreaMomentOfInertia
        """
        self.unit: AreaMomentOfInertiaUnits = unit
        """
        The specific unit that the AreaMomentOfInertia value is representing
        """

    def to_json(self):
        """
        Get a AreaMomentOfInertia DTO JSON object representing the current unit.

        :return: JSON object represents AreaMomentOfInertia DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MeterToTheFourth"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of AreaMomentOfInertia DTO from a json representation.

        :param data: The AreaMomentOfInertia DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MeterToTheFourth"}
        :return: A new instance of AreaMomentOfInertiaDto.
        :rtype: AreaMomentOfInertiaDto
        """
        return AreaMomentOfInertiaDto(value=data["value"], unit=AreaMomentOfInertiaUnits(data["unit"]))


class AreaMomentOfInertia(AbstractMeasure):
    """
    A geometric property of an area that reflects how its points are distributed with regard to an axis.

    Args:
        value (float): The value.
        from_unit (AreaMomentOfInertiaUnits): The AreaMomentOfInertia unit to create from, The default unit is MeterToTheFourth
    """
    def __init__(self, value: float, from_unit: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__meters_to_the_fourth = None
        
        self.__decimeters_to_the_fourth = None
        
        self.__centimeters_to_the_fourth = None
        
        self.__millimeters_to_the_fourth = None
        
        self.__feet_to_the_fourth = None
        
        self.__inches_to_the_fourth = None
        

    def convert(self, unit: AreaMomentOfInertiaUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth) -> AreaMomentOfInertiaDto:
        """
        Get a new instance of AreaMomentOfInertia DTO representing the current unit.

        :param hold_in_unit: The specific AreaMomentOfInertia unit to store the AreaMomentOfInertia value in the DTO representation.
        :type hold_in_unit: AreaMomentOfInertiaUnits
        :return: A new instance of AreaMomentOfInertiaDto.
        :rtype: AreaMomentOfInertiaDto
        """
        return AreaMomentOfInertiaDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth):
        """
        Get a AreaMomentOfInertia DTO JSON object representing the current unit.

        :param hold_in_unit: The specific AreaMomentOfInertia unit to store the AreaMomentOfInertia value in the DTO representation.
        :type hold_in_unit: AreaMomentOfInertiaUnits
        :return: JSON object represents AreaMomentOfInertia DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MeterToTheFourth"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(area_moment_of_inertia_dto: AreaMomentOfInertiaDto):
        """
        Obtain a new instance of AreaMomentOfInertia from a DTO unit object.

        :param area_moment_of_inertia_dto: The AreaMomentOfInertia DTO representation.
        :type area_moment_of_inertia_dto: AreaMomentOfInertiaDto
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(area_moment_of_inertia_dto.value, area_moment_of_inertia_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of AreaMomentOfInertia from a DTO unit json representation.

        :param data: The AreaMomentOfInertia DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MeterToTheFourth"}
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia.from_dto(AreaMomentOfInertiaDto.from_json(data))

    def __convert_from_base(self, from_unit: AreaMomentOfInertiaUnits) -> float:
        value = self._value
        
        if from_unit == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return (value)
        
        if from_unit == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return (value * 1e4)
        
        if from_unit == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return (value * 1e8)
        
        if from_unit == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return (value * 1e12)
        
        if from_unit == AreaMomentOfInertiaUnits.FootToTheFourth:
            return (value / math.pow(0.3048, 4))
        
        if from_unit == AreaMomentOfInertiaUnits.InchToTheFourth:
            return (value / math.pow(2.54e-2, 4))
        
        return None


    def __convert_to_base(self, value: float, to_unit: AreaMomentOfInertiaUnits) -> float:
        
        if to_unit == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return (value)
        
        if to_unit == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return (value / 1e4)
        
        if to_unit == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return (value / 1e8)
        
        if to_unit == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return (value / 1e12)
        
        if to_unit == AreaMomentOfInertiaUnits.FootToTheFourth:
            return (value * math.pow(0.3048, 4))
        
        if to_unit == AreaMomentOfInertiaUnits.InchToTheFourth:
            return (value * math.pow(2.54e-2, 4))
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_meters_to_the_fourth(meters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in meters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in meters_to_the_fourth.
        :type meters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(meters_to_the_fourth, AreaMomentOfInertiaUnits.MeterToTheFourth)

    
    @staticmethod
    def from_decimeters_to_the_fourth(decimeters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in decimeters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in decimeters_to_the_fourth.
        :type decimeters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(decimeters_to_the_fourth, AreaMomentOfInertiaUnits.DecimeterToTheFourth)

    
    @staticmethod
    def from_centimeters_to_the_fourth(centimeters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in centimeters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in centimeters_to_the_fourth.
        :type centimeters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(centimeters_to_the_fourth, AreaMomentOfInertiaUnits.CentimeterToTheFourth)

    
    @staticmethod
    def from_millimeters_to_the_fourth(millimeters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in millimeters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in millimeters_to_the_fourth.
        :type millimeters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(millimeters_to_the_fourth, AreaMomentOfInertiaUnits.MillimeterToTheFourth)

    
    @staticmethod
    def from_feet_to_the_fourth(feet_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in feet_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in feet_to_the_fourth.
        :type feet_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(feet_to_the_fourth, AreaMomentOfInertiaUnits.FootToTheFourth)

    
    @staticmethod
    def from_inches_to_the_fourth(inches_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in inches_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in inches_to_the_fourth.
        :type inches_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(inches_to_the_fourth, AreaMomentOfInertiaUnits.InchToTheFourth)

    
    @property
    def meters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__meters_to_the_fourth != None:
            return self.__meters_to_the_fourth
        self.__meters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.MeterToTheFourth)
        return self.__meters_to_the_fourth

    
    @property
    def decimeters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__decimeters_to_the_fourth != None:
            return self.__decimeters_to_the_fourth
        self.__decimeters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.DecimeterToTheFourth)
        return self.__decimeters_to_the_fourth

    
    @property
    def centimeters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__centimeters_to_the_fourth != None:
            return self.__centimeters_to_the_fourth
        self.__centimeters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.CentimeterToTheFourth)
        return self.__centimeters_to_the_fourth

    
    @property
    def millimeters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__millimeters_to_the_fourth != None:
            return self.__millimeters_to_the_fourth
        self.__millimeters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.MillimeterToTheFourth)
        return self.__millimeters_to_the_fourth

    
    @property
    def feet_to_the_fourth(self) -> float:
        """
        
        """
        if self.__feet_to_the_fourth != None:
            return self.__feet_to_the_fourth
        self.__feet_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.FootToTheFourth)
        return self.__feet_to_the_fourth

    
    @property
    def inches_to_the_fourth(self) -> float:
        """
        
        """
        if self.__inches_to_the_fourth != None:
            return self.__inches_to_the_fourth
        self.__inches_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.InchToTheFourth)
        return self.__inches_to_the_fourth

    
    def to_string(self, unit: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth, fractional_digits: int = None) -> str:
        """
        Format the AreaMomentOfInertia to a string.
        
        Note: the default format for AreaMomentOfInertia is MeterToTheFourth.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the AreaMomentOfInertia. Default is 'MeterToTheFourth'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return f"""{super()._truncate_fraction_digits(self.meters_to_the_fourth, fractional_digits)} m⁴"""
        
        if unit == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return f"""{super()._truncate_fraction_digits(self.decimeters_to_the_fourth, fractional_digits)} dm⁴"""
        
        if unit == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return f"""{super()._truncate_fraction_digits(self.centimeters_to_the_fourth, fractional_digits)} cm⁴"""
        
        if unit == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return f"""{super()._truncate_fraction_digits(self.millimeters_to_the_fourth, fractional_digits)} mm⁴"""
        
        if unit == AreaMomentOfInertiaUnits.FootToTheFourth:
            return f"""{super()._truncate_fraction_digits(self.feet_to_the_fourth, fractional_digits)} ft⁴"""
        
        if unit == AreaMomentOfInertiaUnits.InchToTheFourth:
            return f"""{super()._truncate_fraction_digits(self.inches_to_the_fourth, fractional_digits)} in⁴"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth) -> str:
        """
        Get AreaMomentOfInertia unit abbreviation.
        Note! the default abbreviation for AreaMomentOfInertia is MeterToTheFourth.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return """m⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return """dm⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return """cm⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return """mm⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.FootToTheFourth:
            return """ft⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.InchToTheFourth:
            return """in⁴"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricPotentialUnits(Enum):
        """
            ElectricPotentialUnits enumeration
        """
        
        Volt = 'Volt'
        """
            
        """
        
        Nanovolt = 'Nanovolt'
        """
            
        """
        
        Microvolt = 'Microvolt'
        """
            
        """
        
        Millivolt = 'Millivolt'
        """
            
        """
        
        Kilovolt = 'Kilovolt'
        """
            
        """
        
        Megavolt = 'Megavolt'
        """
            
        """
        

class ElectricPotentialDto:
    """
    A DTO representation of a ElectricPotential

    Attributes:
        value (float): The value of the ElectricPotential.
        unit (ElectricPotentialUnits): The specific unit that the ElectricPotential value is representing.
    """

    def __init__(self, value: float, unit: ElectricPotentialUnits):
        """
        Create a new DTO representation of a ElectricPotential

        Parameters:
            value (float): The value of the ElectricPotential.
            unit (ElectricPotentialUnits): The specific unit that the ElectricPotential value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricPotential
        """
        self.unit: ElectricPotentialUnits = unit
        """
        The specific unit that the ElectricPotential value is representing
        """

    def to_json(self):
        """
        Get a ElectricPotential DTO JSON object representing the current unit.

        :return: JSON object represents ElectricPotential DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Volt"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricPotential DTO from a json representation.

        :param data: The ElectricPotential DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Volt"}
        :return: A new instance of ElectricPotentialDto.
        :rtype: ElectricPotentialDto
        """
        return ElectricPotentialDto(value=data["value"], unit=ElectricPotentialUnits(data["unit"]))


class ElectricPotential(AbstractMeasure):
    """
    In classical electromagnetism, the electric potential (a scalar quantity denoted by Φ, ΦE or V and also called the electric field potential or the electrostatic potential) at a point is the amount of electric potential energy that a unitary point charge would have when located at that point.

    Args:
        value (float): The value.
        from_unit (ElectricPotentialUnits): The ElectricPotential unit to create from, The default unit is Volt
    """
    def __init__(self, value: float, from_unit: ElectricPotentialUnits = ElectricPotentialUnits.Volt):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__volts = None
        
        self.__nanovolts = None
        
        self.__microvolts = None
        
        self.__millivolts = None
        
        self.__kilovolts = None
        
        self.__megavolts = None
        

    def convert(self, unit: ElectricPotentialUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricPotentialUnits = ElectricPotentialUnits.Volt) -> ElectricPotentialDto:
        """
        Get a new instance of ElectricPotential DTO representing the current unit.

        :param hold_in_unit: The specific ElectricPotential unit to store the ElectricPotential value in the DTO representation.
        :type hold_in_unit: ElectricPotentialUnits
        :return: A new instance of ElectricPotentialDto.
        :rtype: ElectricPotentialDto
        """
        return ElectricPotentialDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricPotentialUnits = ElectricPotentialUnits.Volt):
        """
        Get a ElectricPotential DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricPotential unit to store the ElectricPotential value in the DTO representation.
        :type hold_in_unit: ElectricPotentialUnits
        :return: JSON object represents ElectricPotential DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Volt"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_potential_dto: ElectricPotentialDto):
        """
        Obtain a new instance of ElectricPotential from a DTO unit object.

        :param electric_potential_dto: The ElectricPotential DTO representation.
        :type electric_potential_dto: ElectricPotentialDto
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential(electric_potential_dto.value, electric_potential_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricPotential from a DTO unit json representation.

        :param data: The ElectricPotential DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Volt"}
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential.from_dto(ElectricPotentialDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricPotentialUnits) -> float:
        value = self._value
        
        if from_unit == ElectricPotentialUnits.Volt:
            return (value)
        
        if from_unit == ElectricPotentialUnits.Nanovolt:
            return ((value) / 1e-09)
        
        if from_unit == ElectricPotentialUnits.Microvolt:
            return ((value) / 1e-06)
        
        if from_unit == ElectricPotentialUnits.Millivolt:
            return ((value) / 0.001)
        
        if from_unit == ElectricPotentialUnits.Kilovolt:
            return ((value) / 1000.0)
        
        if from_unit == ElectricPotentialUnits.Megavolt:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricPotentialUnits) -> float:
        
        if to_unit == ElectricPotentialUnits.Volt:
            return (value)
        
        if to_unit == ElectricPotentialUnits.Nanovolt:
            return ((value) * 1e-09)
        
        if to_unit == ElectricPotentialUnits.Microvolt:
            return ((value) * 1e-06)
        
        if to_unit == ElectricPotentialUnits.Millivolt:
            return ((value) * 0.001)
        
        if to_unit == ElectricPotentialUnits.Kilovolt:
            return ((value) * 1000.0)
        
        if to_unit == ElectricPotentialUnits.Megavolt:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_volts(volts: float):
        """
        Create a new instance of ElectricPotential from a value in volts.

        

        :param meters: The ElectricPotential value in volts.
        :type volts: float
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential(volts, ElectricPotentialUnits.Volt)

    
    @staticmethod
    def from_nanovolts(nanovolts: float):
        """
        Create a new instance of ElectricPotential from a value in nanovolts.

        

        :param meters: The ElectricPotential value in nanovolts.
        :type nanovolts: float
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential(nanovolts, ElectricPotentialUnits.Nanovolt)

    
    @staticmethod
    def from_microvolts(microvolts: float):
        """
        Create a new instance of ElectricPotential from a value in microvolts.

        

        :param meters: The ElectricPotential value in microvolts.
        :type microvolts: float
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential(microvolts, ElectricPotentialUnits.Microvolt)

    
    @staticmethod
    def from_millivolts(millivolts: float):
        """
        Create a new instance of ElectricPotential from a value in millivolts.

        

        :param meters: The ElectricPotential value in millivolts.
        :type millivolts: float
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential(millivolts, ElectricPotentialUnits.Millivolt)

    
    @staticmethod
    def from_kilovolts(kilovolts: float):
        """
        Create a new instance of ElectricPotential from a value in kilovolts.

        

        :param meters: The ElectricPotential value in kilovolts.
        :type kilovolts: float
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential(kilovolts, ElectricPotentialUnits.Kilovolt)

    
    @staticmethod
    def from_megavolts(megavolts: float):
        """
        Create a new instance of ElectricPotential from a value in megavolts.

        

        :param meters: The ElectricPotential value in megavolts.
        :type megavolts: float
        :return: A new instance of ElectricPotential.
        :rtype: ElectricPotential
        """
        return ElectricPotential(megavolts, ElectricPotentialUnits.Megavolt)

    
    @property
    def volts(self) -> float:
        """
        
        """
        if self.__volts != None:
            return self.__volts
        self.__volts = self.__convert_from_base(ElectricPotentialUnits.Volt)
        return self.__volts

    
    @property
    def nanovolts(self) -> float:
        """
        
        """
        if self.__nanovolts != None:
            return self.__nanovolts
        self.__nanovolts = self.__convert_from_base(ElectricPotentialUnits.Nanovolt)
        return self.__nanovolts

    
    @property
    def microvolts(self) -> float:
        """
        
        """
        if self.__microvolts != None:
            return self.__microvolts
        self.__microvolts = self.__convert_from_base(ElectricPotentialUnits.Microvolt)
        return self.__microvolts

    
    @property
    def millivolts(self) -> float:
        """
        
        """
        if self.__millivolts != None:
            return self.__millivolts
        self.__millivolts = self.__convert_from_base(ElectricPotentialUnits.Millivolt)
        return self.__millivolts

    
    @property
    def kilovolts(self) -> float:
        """
        
        """
        if self.__kilovolts != None:
            return self.__kilovolts
        self.__kilovolts = self.__convert_from_base(ElectricPotentialUnits.Kilovolt)
        return self.__kilovolts

    
    @property
    def megavolts(self) -> float:
        """
        
        """
        if self.__megavolts != None:
            return self.__megavolts
        self.__megavolts = self.__convert_from_base(ElectricPotentialUnits.Megavolt)
        return self.__megavolts

    
    def to_string(self, unit: ElectricPotentialUnits = ElectricPotentialUnits.Volt, fractional_digits: int = None) -> str:
        """
        Format the ElectricPotential to a string.
        
        Note: the default format for ElectricPotential is Volt.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricPotential. Default is 'Volt'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricPotentialUnits.Volt:
            return f"""{super()._truncate_fraction_digits(self.volts, fractional_digits)} V"""
        
        if unit == ElectricPotentialUnits.Nanovolt:
            return f"""{super()._truncate_fraction_digits(self.nanovolts, fractional_digits)} nV"""
        
        if unit == ElectricPotentialUnits.Microvolt:
            return f"""{super()._truncate_fraction_digits(self.microvolts, fractional_digits)} μV"""
        
        if unit == ElectricPotentialUnits.Millivolt:
            return f"""{super()._truncate_fraction_digits(self.millivolts, fractional_digits)} mV"""
        
        if unit == ElectricPotentialUnits.Kilovolt:
            return f"""{super()._truncate_fraction_digits(self.kilovolts, fractional_digits)} kV"""
        
        if unit == ElectricPotentialUnits.Megavolt:
            return f"""{super()._truncate_fraction_digits(self.megavolts, fractional_digits)} MV"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricPotentialUnits = ElectricPotentialUnits.Volt) -> str:
        """
        Get ElectricPotential unit abbreviation.
        Note! the default abbreviation for ElectricPotential is Volt.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricPotentialUnits.Volt:
            return """V"""
        
        if unit_abbreviation == ElectricPotentialUnits.Nanovolt:
            return """nV"""
        
        if unit_abbreviation == ElectricPotentialUnits.Microvolt:
            return """μV"""
        
        if unit_abbreviation == ElectricPotentialUnits.Millivolt:
            return """mV"""
        
        if unit_abbreviation == ElectricPotentialUnits.Kilovolt:
            return """kV"""
        
        if unit_abbreviation == ElectricPotentialUnits.Megavolt:
            return """MV"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class TorqueUnits(Enum):
        """
            TorqueUnits enumeration
        """
        
        NewtonMillimeter = 'NewtonMillimeter'
        """
            
        """
        
        NewtonCentimeter = 'NewtonCentimeter'
        """
            
        """
        
        NewtonMeter = 'NewtonMeter'
        """
            
        """
        
        PoundalFoot = 'PoundalFoot'
        """
            
        """
        
        PoundForceInch = 'PoundForceInch'
        """
            
        """
        
        PoundForceFoot = 'PoundForceFoot'
        """
            
        """
        
        GramForceMillimeter = 'GramForceMillimeter'
        """
            
        """
        
        GramForceCentimeter = 'GramForceCentimeter'
        """
            
        """
        
        GramForceMeter = 'GramForceMeter'
        """
            
        """
        
        KilogramForceMillimeter = 'KilogramForceMillimeter'
        """
            
        """
        
        KilogramForceCentimeter = 'KilogramForceCentimeter'
        """
            
        """
        
        KilogramForceMeter = 'KilogramForceMeter'
        """
            
        """
        
        TonneForceMillimeter = 'TonneForceMillimeter'
        """
            
        """
        
        TonneForceCentimeter = 'TonneForceCentimeter'
        """
            
        """
        
        TonneForceMeter = 'TonneForceMeter'
        """
            
        """
        
        KilonewtonMillimeter = 'KilonewtonMillimeter'
        """
            
        """
        
        MeganewtonMillimeter = 'MeganewtonMillimeter'
        """
            
        """
        
        KilonewtonCentimeter = 'KilonewtonCentimeter'
        """
            
        """
        
        MeganewtonCentimeter = 'MeganewtonCentimeter'
        """
            
        """
        
        KilonewtonMeter = 'KilonewtonMeter'
        """
            
        """
        
        MeganewtonMeter = 'MeganewtonMeter'
        """
            
        """
        
        KilopoundForceInch = 'KilopoundForceInch'
        """
            
        """
        
        MegapoundForceInch = 'MegapoundForceInch'
        """
            
        """
        
        KilopoundForceFoot = 'KilopoundForceFoot'
        """
            
        """
        
        MegapoundForceFoot = 'MegapoundForceFoot'
        """
            
        """
        

class TorqueDto:
    """
    A DTO representation of a Torque

    Attributes:
        value (float): The value of the Torque.
        unit (TorqueUnits): The specific unit that the Torque value is representing.
    """

    def __init__(self, value: float, unit: TorqueUnits):
        """
        Create a new DTO representation of a Torque

        Parameters:
            value (float): The value of the Torque.
            unit (TorqueUnits): The specific unit that the Torque value is representing.
        """
        self.value: float = value
        """
        The value of the Torque
        """
        self.unit: TorqueUnits = unit
        """
        The specific unit that the Torque value is representing
        """

    def to_json(self):
        """
        Get a Torque DTO JSON object representing the current unit.

        :return: JSON object represents Torque DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Torque DTO from a json representation.

        :param data: The Torque DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonMeter"}
        :return: A new instance of TorqueDto.
        :rtype: TorqueDto
        """
        return TorqueDto(value=data["value"], unit=TorqueUnits(data["unit"]))


class Torque(AbstractMeasure):
    """
    Torque, moment or moment of force (see the terminology below), is the tendency of a force to rotate an object about an axis,[1] fulcrum, or pivot. Just as a force is a push or a pull, a torque can be thought of as a twist to an object. Mathematically, torque is defined as the cross product of the lever-arm distance and force, which tends to produce rotation. Loosely speaking, torque is a measure of the turning force on an object such as a bolt or a flywheel. For example, pushing or pulling the handle of a wrench connected to a nut or bolt produces a torque (turning force) that loosens or tightens the nut or bolt.

    Args:
        value (float): The value.
        from_unit (TorqueUnits): The Torque unit to create from, The default unit is NewtonMeter
    """
    def __init__(self, value: float, from_unit: TorqueUnits = TorqueUnits.NewtonMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__newton_millimeters = None
        
        self.__newton_centimeters = None
        
        self.__newton_meters = None
        
        self.__poundal_feet = None
        
        self.__pound_force_inches = None
        
        self.__pound_force_feet = None
        
        self.__gram_force_millimeters = None
        
        self.__gram_force_centimeters = None
        
        self.__gram_force_meters = None
        
        self.__kilogram_force_millimeters = None
        
        self.__kilogram_force_centimeters = None
        
        self.__kilogram_force_meters = None
        
        self.__tonne_force_millimeters = None
        
        self.__tonne_force_centimeters = None
        
        self.__tonne_force_meters = None
        
        self.__kilonewton_millimeters = None
        
        self.__meganewton_millimeters = None
        
        self.__kilonewton_centimeters = None
        
        self.__meganewton_centimeters = None
        
        self.__kilonewton_meters = None
        
        self.__meganewton_meters = None
        
        self.__kilopound_force_inches = None
        
        self.__megapound_force_inches = None
        
        self.__kilopound_force_feet = None
        
        self.__megapound_force_feet = None
        

    def convert(self, unit: TorqueUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: TorqueUnits = TorqueUnits.NewtonMeter) -> TorqueDto:
        """
        Get a new instance of Torque DTO representing the current unit.

        :param hold_in_unit: The specific Torque unit to store the Torque value in the DTO representation.
        :type hold_in_unit: TorqueUnits
        :return: A new instance of TorqueDto.
        :rtype: TorqueDto
        """
        return TorqueDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: TorqueUnits = TorqueUnits.NewtonMeter):
        """
        Get a Torque DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Torque unit to store the Torque value in the DTO representation.
        :type hold_in_unit: TorqueUnits
        :return: JSON object represents Torque DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(torque_dto: TorqueDto):
        """
        Obtain a new instance of Torque from a DTO unit object.

        :param torque_dto: The Torque DTO representation.
        :type torque_dto: TorqueDto
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(torque_dto.value, torque_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Torque from a DTO unit json representation.

        :param data: The Torque DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonMeter"}
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque.from_dto(TorqueDto.from_json(data))

    def __convert_from_base(self, from_unit: TorqueUnits) -> float:
        value = self._value
        
        if from_unit == TorqueUnits.NewtonMillimeter:
            return (value * 1000)
        
        if from_unit == TorqueUnits.NewtonCentimeter:
            return (value * 100)
        
        if from_unit == TorqueUnits.NewtonMeter:
            return (value)
        
        if from_unit == TorqueUnits.PoundalFoot:
            return (value / 4.21401100938048e-2)
        
        if from_unit == TorqueUnits.PoundForceInch:
            return (value / 1.129848290276167e-1)
        
        if from_unit == TorqueUnits.PoundForceFoot:
            return (value / 1.3558179483314)
        
        if from_unit == TorqueUnits.GramForceMillimeter:
            return (value / 9.80665e-6)
        
        if from_unit == TorqueUnits.GramForceCentimeter:
            return (value / 9.80665e-5)
        
        if from_unit == TorqueUnits.GramForceMeter:
            return (value / 9.80665e-3)
        
        if from_unit == TorqueUnits.KilogramForceMillimeter:
            return (value / 9.80665e-3)
        
        if from_unit == TorqueUnits.KilogramForceCentimeter:
            return (value / 9.80665e-2)
        
        if from_unit == TorqueUnits.KilogramForceMeter:
            return (value / 9.80665)
        
        if from_unit == TorqueUnits.TonneForceMillimeter:
            return (value / 9.80665)
        
        if from_unit == TorqueUnits.TonneForceCentimeter:
            return (value / 9.80665e1)
        
        if from_unit == TorqueUnits.TonneForceMeter:
            return (value / 9.80665e3)
        
        if from_unit == TorqueUnits.KilonewtonMillimeter:
            return ((value * 1000) / 1000.0)
        
        if from_unit == TorqueUnits.MeganewtonMillimeter:
            return ((value * 1000) / 1000000.0)
        
        if from_unit == TorqueUnits.KilonewtonCentimeter:
            return ((value * 100) / 1000.0)
        
        if from_unit == TorqueUnits.MeganewtonCentimeter:
            return ((value * 100) / 1000000.0)
        
        if from_unit == TorqueUnits.KilonewtonMeter:
            return ((value) / 1000.0)
        
        if from_unit == TorqueUnits.MeganewtonMeter:
            return ((value) / 1000000.0)
        
        if from_unit == TorqueUnits.KilopoundForceInch:
            return ((value / 1.129848290276167e-1) / 1000.0)
        
        if from_unit == TorqueUnits.MegapoundForceInch:
            return ((value / 1.129848290276167e-1) / 1000000.0)
        
        if from_unit == TorqueUnits.KilopoundForceFoot:
            return ((value / 1.3558179483314) / 1000.0)
        
        if from_unit == TorqueUnits.MegapoundForceFoot:
            return ((value / 1.3558179483314) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TorqueUnits) -> float:
        
        if to_unit == TorqueUnits.NewtonMillimeter:
            return (value * 0.001)
        
        if to_unit == TorqueUnits.NewtonCentimeter:
            return (value * 0.01)
        
        if to_unit == TorqueUnits.NewtonMeter:
            return (value)
        
        if to_unit == TorqueUnits.PoundalFoot:
            return (value * 4.21401100938048e-2)
        
        if to_unit == TorqueUnits.PoundForceInch:
            return (value * 1.129848290276167e-1)
        
        if to_unit == TorqueUnits.PoundForceFoot:
            return (value * 1.3558179483314)
        
        if to_unit == TorqueUnits.GramForceMillimeter:
            return (value * 9.80665e-6)
        
        if to_unit == TorqueUnits.GramForceCentimeter:
            return (value * 9.80665e-5)
        
        if to_unit == TorqueUnits.GramForceMeter:
            return (value * 9.80665e-3)
        
        if to_unit == TorqueUnits.KilogramForceMillimeter:
            return (value * 9.80665e-3)
        
        if to_unit == TorqueUnits.KilogramForceCentimeter:
            return (value * 9.80665e-2)
        
        if to_unit == TorqueUnits.KilogramForceMeter:
            return (value * 9.80665)
        
        if to_unit == TorqueUnits.TonneForceMillimeter:
            return (value * 9.80665)
        
        if to_unit == TorqueUnits.TonneForceCentimeter:
            return (value * 9.80665e1)
        
        if to_unit == TorqueUnits.TonneForceMeter:
            return (value * 9.80665e3)
        
        if to_unit == TorqueUnits.KilonewtonMillimeter:
            return ((value * 0.001) * 1000.0)
        
        if to_unit == TorqueUnits.MeganewtonMillimeter:
            return ((value * 0.001) * 1000000.0)
        
        if to_unit == TorqueUnits.KilonewtonCentimeter:
            return ((value * 0.01) * 1000.0)
        
        if to_unit == TorqueUnits.MeganewtonCentimeter:
            return ((value * 0.01) * 1000000.0)
        
        if to_unit == TorqueUnits.KilonewtonMeter:
            return ((value) * 1000.0)
        
        if to_unit == TorqueUnits.MeganewtonMeter:
            return ((value) * 1000000.0)
        
        if to_unit == TorqueUnits.KilopoundForceInch:
            return ((value * 1.129848290276167e-1) * 1000.0)
        
        if to_unit == TorqueUnits.MegapoundForceInch:
            return ((value * 1.129848290276167e-1) * 1000000.0)
        
        if to_unit == TorqueUnits.KilopoundForceFoot:
            return ((value * 1.3558179483314) * 1000.0)
        
        if to_unit == TorqueUnits.MegapoundForceFoot:
            return ((value * 1.3558179483314) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_newton_millimeters(newton_millimeters: float):
        """
        Create a new instance of Torque from a value in newton_millimeters.

        

        :param meters: The Torque value in newton_millimeters.
        :type newton_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(newton_millimeters, TorqueUnits.NewtonMillimeter)

    
    @staticmethod
    def from_newton_centimeters(newton_centimeters: float):
        """
        Create a new instance of Torque from a value in newton_centimeters.

        

        :param meters: The Torque value in newton_centimeters.
        :type newton_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(newton_centimeters, TorqueUnits.NewtonCentimeter)

    
    @staticmethod
    def from_newton_meters(newton_meters: float):
        """
        Create a new instance of Torque from a value in newton_meters.

        

        :param meters: The Torque value in newton_meters.
        :type newton_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(newton_meters, TorqueUnits.NewtonMeter)

    
    @staticmethod
    def from_poundal_feet(poundal_feet: float):
        """
        Create a new instance of Torque from a value in poundal_feet.

        

        :param meters: The Torque value in poundal_feet.
        :type poundal_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(poundal_feet, TorqueUnits.PoundalFoot)

    
    @staticmethod
    def from_pound_force_inches(pound_force_inches: float):
        """
        Create a new instance of Torque from a value in pound_force_inches.

        

        :param meters: The Torque value in pound_force_inches.
        :type pound_force_inches: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(pound_force_inches, TorqueUnits.PoundForceInch)

    
    @staticmethod
    def from_pound_force_feet(pound_force_feet: float):
        """
        Create a new instance of Torque from a value in pound_force_feet.

        

        :param meters: The Torque value in pound_force_feet.
        :type pound_force_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(pound_force_feet, TorqueUnits.PoundForceFoot)

    
    @staticmethod
    def from_gram_force_millimeters(gram_force_millimeters: float):
        """
        Create a new instance of Torque from a value in gram_force_millimeters.

        

        :param meters: The Torque value in gram_force_millimeters.
        :type gram_force_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(gram_force_millimeters, TorqueUnits.GramForceMillimeter)

    
    @staticmethod
    def from_gram_force_centimeters(gram_force_centimeters: float):
        """
        Create a new instance of Torque from a value in gram_force_centimeters.

        

        :param meters: The Torque value in gram_force_centimeters.
        :type gram_force_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(gram_force_centimeters, TorqueUnits.GramForceCentimeter)

    
    @staticmethod
    def from_gram_force_meters(gram_force_meters: float):
        """
        Create a new instance of Torque from a value in gram_force_meters.

        

        :param meters: The Torque value in gram_force_meters.
        :type gram_force_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(gram_force_meters, TorqueUnits.GramForceMeter)

    
    @staticmethod
    def from_kilogram_force_millimeters(kilogram_force_millimeters: float):
        """
        Create a new instance of Torque from a value in kilogram_force_millimeters.

        

        :param meters: The Torque value in kilogram_force_millimeters.
        :type kilogram_force_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilogram_force_millimeters, TorqueUnits.KilogramForceMillimeter)

    
    @staticmethod
    def from_kilogram_force_centimeters(kilogram_force_centimeters: float):
        """
        Create a new instance of Torque from a value in kilogram_force_centimeters.

        

        :param meters: The Torque value in kilogram_force_centimeters.
        :type kilogram_force_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilogram_force_centimeters, TorqueUnits.KilogramForceCentimeter)

    
    @staticmethod
    def from_kilogram_force_meters(kilogram_force_meters: float):
        """
        Create a new instance of Torque from a value in kilogram_force_meters.

        

        :param meters: The Torque value in kilogram_force_meters.
        :type kilogram_force_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilogram_force_meters, TorqueUnits.KilogramForceMeter)

    
    @staticmethod
    def from_tonne_force_millimeters(tonne_force_millimeters: float):
        """
        Create a new instance of Torque from a value in tonne_force_millimeters.

        

        :param meters: The Torque value in tonne_force_millimeters.
        :type tonne_force_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(tonne_force_millimeters, TorqueUnits.TonneForceMillimeter)

    
    @staticmethod
    def from_tonne_force_centimeters(tonne_force_centimeters: float):
        """
        Create a new instance of Torque from a value in tonne_force_centimeters.

        

        :param meters: The Torque value in tonne_force_centimeters.
        :type tonne_force_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(tonne_force_centimeters, TorqueUnits.TonneForceCentimeter)

    
    @staticmethod
    def from_tonne_force_meters(tonne_force_meters: float):
        """
        Create a new instance of Torque from a value in tonne_force_meters.

        

        :param meters: The Torque value in tonne_force_meters.
        :type tonne_force_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(tonne_force_meters, TorqueUnits.TonneForceMeter)

    
    @staticmethod
    def from_kilonewton_millimeters(kilonewton_millimeters: float):
        """
        Create a new instance of Torque from a value in kilonewton_millimeters.

        

        :param meters: The Torque value in kilonewton_millimeters.
        :type kilonewton_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilonewton_millimeters, TorqueUnits.KilonewtonMillimeter)

    
    @staticmethod
    def from_meganewton_millimeters(meganewton_millimeters: float):
        """
        Create a new instance of Torque from a value in meganewton_millimeters.

        

        :param meters: The Torque value in meganewton_millimeters.
        :type meganewton_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(meganewton_millimeters, TorqueUnits.MeganewtonMillimeter)

    
    @staticmethod
    def from_kilonewton_centimeters(kilonewton_centimeters: float):
        """
        Create a new instance of Torque from a value in kilonewton_centimeters.

        

        :param meters: The Torque value in kilonewton_centimeters.
        :type kilonewton_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilonewton_centimeters, TorqueUnits.KilonewtonCentimeter)

    
    @staticmethod
    def from_meganewton_centimeters(meganewton_centimeters: float):
        """
        Create a new instance of Torque from a value in meganewton_centimeters.

        

        :param meters: The Torque value in meganewton_centimeters.
        :type meganewton_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(meganewton_centimeters, TorqueUnits.MeganewtonCentimeter)

    
    @staticmethod
    def from_kilonewton_meters(kilonewton_meters: float):
        """
        Create a new instance of Torque from a value in kilonewton_meters.

        

        :param meters: The Torque value in kilonewton_meters.
        :type kilonewton_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilonewton_meters, TorqueUnits.KilonewtonMeter)

    
    @staticmethod
    def from_meganewton_meters(meganewton_meters: float):
        """
        Create a new instance of Torque from a value in meganewton_meters.

        

        :param meters: The Torque value in meganewton_meters.
        :type meganewton_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(meganewton_meters, TorqueUnits.MeganewtonMeter)

    
    @staticmethod
    def from_kilopound_force_inches(kilopound_force_inches: float):
        """
        Create a new instance of Torque from a value in kilopound_force_inches.

        

        :param meters: The Torque value in kilopound_force_inches.
        :type kilopound_force_inches: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilopound_force_inches, TorqueUnits.KilopoundForceInch)

    
    @staticmethod
    def from_megapound_force_inches(megapound_force_inches: float):
        """
        Create a new instance of Torque from a value in megapound_force_inches.

        

        :param meters: The Torque value in megapound_force_inches.
        :type megapound_force_inches: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(megapound_force_inches, TorqueUnits.MegapoundForceInch)

    
    @staticmethod
    def from_kilopound_force_feet(kilopound_force_feet: float):
        """
        Create a new instance of Torque from a value in kilopound_force_feet.

        

        :param meters: The Torque value in kilopound_force_feet.
        :type kilopound_force_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilopound_force_feet, TorqueUnits.KilopoundForceFoot)

    
    @staticmethod
    def from_megapound_force_feet(megapound_force_feet: float):
        """
        Create a new instance of Torque from a value in megapound_force_feet.

        

        :param meters: The Torque value in megapound_force_feet.
        :type megapound_force_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(megapound_force_feet, TorqueUnits.MegapoundForceFoot)

    
    @property
    def newton_millimeters(self) -> float:
        """
        
        """
        if self.__newton_millimeters != None:
            return self.__newton_millimeters
        self.__newton_millimeters = self.__convert_from_base(TorqueUnits.NewtonMillimeter)
        return self.__newton_millimeters

    
    @property
    def newton_centimeters(self) -> float:
        """
        
        """
        if self.__newton_centimeters != None:
            return self.__newton_centimeters
        self.__newton_centimeters = self.__convert_from_base(TorqueUnits.NewtonCentimeter)
        return self.__newton_centimeters

    
    @property
    def newton_meters(self) -> float:
        """
        
        """
        if self.__newton_meters != None:
            return self.__newton_meters
        self.__newton_meters = self.__convert_from_base(TorqueUnits.NewtonMeter)
        return self.__newton_meters

    
    @property
    def poundal_feet(self) -> float:
        """
        
        """
        if self.__poundal_feet != None:
            return self.__poundal_feet
        self.__poundal_feet = self.__convert_from_base(TorqueUnits.PoundalFoot)
        return self.__poundal_feet

    
    @property
    def pound_force_inches(self) -> float:
        """
        
        """
        if self.__pound_force_inches != None:
            return self.__pound_force_inches
        self.__pound_force_inches = self.__convert_from_base(TorqueUnits.PoundForceInch)
        return self.__pound_force_inches

    
    @property
    def pound_force_feet(self) -> float:
        """
        
        """
        if self.__pound_force_feet != None:
            return self.__pound_force_feet
        self.__pound_force_feet = self.__convert_from_base(TorqueUnits.PoundForceFoot)
        return self.__pound_force_feet

    
    @property
    def gram_force_millimeters(self) -> float:
        """
        
        """
        if self.__gram_force_millimeters != None:
            return self.__gram_force_millimeters
        self.__gram_force_millimeters = self.__convert_from_base(TorqueUnits.GramForceMillimeter)
        return self.__gram_force_millimeters

    
    @property
    def gram_force_centimeters(self) -> float:
        """
        
        """
        if self.__gram_force_centimeters != None:
            return self.__gram_force_centimeters
        self.__gram_force_centimeters = self.__convert_from_base(TorqueUnits.GramForceCentimeter)
        return self.__gram_force_centimeters

    
    @property
    def gram_force_meters(self) -> float:
        """
        
        """
        if self.__gram_force_meters != None:
            return self.__gram_force_meters
        self.__gram_force_meters = self.__convert_from_base(TorqueUnits.GramForceMeter)
        return self.__gram_force_meters

    
    @property
    def kilogram_force_millimeters(self) -> float:
        """
        
        """
        if self.__kilogram_force_millimeters != None:
            return self.__kilogram_force_millimeters
        self.__kilogram_force_millimeters = self.__convert_from_base(TorqueUnits.KilogramForceMillimeter)
        return self.__kilogram_force_millimeters

    
    @property
    def kilogram_force_centimeters(self) -> float:
        """
        
        """
        if self.__kilogram_force_centimeters != None:
            return self.__kilogram_force_centimeters
        self.__kilogram_force_centimeters = self.__convert_from_base(TorqueUnits.KilogramForceCentimeter)
        return self.__kilogram_force_centimeters

    
    @property
    def kilogram_force_meters(self) -> float:
        """
        
        """
        if self.__kilogram_force_meters != None:
            return self.__kilogram_force_meters
        self.__kilogram_force_meters = self.__convert_from_base(TorqueUnits.KilogramForceMeter)
        return self.__kilogram_force_meters

    
    @property
    def tonne_force_millimeters(self) -> float:
        """
        
        """
        if self.__tonne_force_millimeters != None:
            return self.__tonne_force_millimeters
        self.__tonne_force_millimeters = self.__convert_from_base(TorqueUnits.TonneForceMillimeter)
        return self.__tonne_force_millimeters

    
    @property
    def tonne_force_centimeters(self) -> float:
        """
        
        """
        if self.__tonne_force_centimeters != None:
            return self.__tonne_force_centimeters
        self.__tonne_force_centimeters = self.__convert_from_base(TorqueUnits.TonneForceCentimeter)
        return self.__tonne_force_centimeters

    
    @property
    def tonne_force_meters(self) -> float:
        """
        
        """
        if self.__tonne_force_meters != None:
            return self.__tonne_force_meters
        self.__tonne_force_meters = self.__convert_from_base(TorqueUnits.TonneForceMeter)
        return self.__tonne_force_meters

    
    @property
    def kilonewton_millimeters(self) -> float:
        """
        
        """
        if self.__kilonewton_millimeters != None:
            return self.__kilonewton_millimeters
        self.__kilonewton_millimeters = self.__convert_from_base(TorqueUnits.KilonewtonMillimeter)
        return self.__kilonewton_millimeters

    
    @property
    def meganewton_millimeters(self) -> float:
        """
        
        """
        if self.__meganewton_millimeters != None:
            return self.__meganewton_millimeters
        self.__meganewton_millimeters = self.__convert_from_base(TorqueUnits.MeganewtonMillimeter)
        return self.__meganewton_millimeters

    
    @property
    def kilonewton_centimeters(self) -> float:
        """
        
        """
        if self.__kilonewton_centimeters != None:
            return self.__kilonewton_centimeters
        self.__kilonewton_centimeters = self.__convert_from_base(TorqueUnits.KilonewtonCentimeter)
        return self.__kilonewton_centimeters

    
    @property
    def meganewton_centimeters(self) -> float:
        """
        
        """
        if self.__meganewton_centimeters != None:
            return self.__meganewton_centimeters
        self.__meganewton_centimeters = self.__convert_from_base(TorqueUnits.MeganewtonCentimeter)
        return self.__meganewton_centimeters

    
    @property
    def kilonewton_meters(self) -> float:
        """
        
        """
        if self.__kilonewton_meters != None:
            return self.__kilonewton_meters
        self.__kilonewton_meters = self.__convert_from_base(TorqueUnits.KilonewtonMeter)
        return self.__kilonewton_meters

    
    @property
    def meganewton_meters(self) -> float:
        """
        
        """
        if self.__meganewton_meters != None:
            return self.__meganewton_meters
        self.__meganewton_meters = self.__convert_from_base(TorqueUnits.MeganewtonMeter)
        return self.__meganewton_meters

    
    @property
    def kilopound_force_inches(self) -> float:
        """
        
        """
        if self.__kilopound_force_inches != None:
            return self.__kilopound_force_inches
        self.__kilopound_force_inches = self.__convert_from_base(TorqueUnits.KilopoundForceInch)
        return self.__kilopound_force_inches

    
    @property
    def megapound_force_inches(self) -> float:
        """
        
        """
        if self.__megapound_force_inches != None:
            return self.__megapound_force_inches
        self.__megapound_force_inches = self.__convert_from_base(TorqueUnits.MegapoundForceInch)
        return self.__megapound_force_inches

    
    @property
    def kilopound_force_feet(self) -> float:
        """
        
        """
        if self.__kilopound_force_feet != None:
            return self.__kilopound_force_feet
        self.__kilopound_force_feet = self.__convert_from_base(TorqueUnits.KilopoundForceFoot)
        return self.__kilopound_force_feet

    
    @property
    def megapound_force_feet(self) -> float:
        """
        
        """
        if self.__megapound_force_feet != None:
            return self.__megapound_force_feet
        self.__megapound_force_feet = self.__convert_from_base(TorqueUnits.MegapoundForceFoot)
        return self.__megapound_force_feet

    
    def to_string(self, unit: TorqueUnits = TorqueUnits.NewtonMeter, fractional_digits: int = None) -> str:
        """
        Format the Torque to a string.
        
        Note: the default format for Torque is NewtonMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Torque. Default is 'NewtonMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == TorqueUnits.NewtonMillimeter:
            return f"""{super()._truncate_fraction_digits(self.newton_millimeters, fractional_digits)} N·mm"""
        
        if unit == TorqueUnits.NewtonCentimeter:
            return f"""{super()._truncate_fraction_digits(self.newton_centimeters, fractional_digits)} N·cm"""
        
        if unit == TorqueUnits.NewtonMeter:
            return f"""{super()._truncate_fraction_digits(self.newton_meters, fractional_digits)} N·m"""
        
        if unit == TorqueUnits.PoundalFoot:
            return f"""{super()._truncate_fraction_digits(self.poundal_feet, fractional_digits)} pdl·ft"""
        
        if unit == TorqueUnits.PoundForceInch:
            return f"""{super()._truncate_fraction_digits(self.pound_force_inches, fractional_digits)} lbf·in"""
        
        if unit == TorqueUnits.PoundForceFoot:
            return f"""{super()._truncate_fraction_digits(self.pound_force_feet, fractional_digits)} lbf·ft"""
        
        if unit == TorqueUnits.GramForceMillimeter:
            return f"""{super()._truncate_fraction_digits(self.gram_force_millimeters, fractional_digits)} gf·mm"""
        
        if unit == TorqueUnits.GramForceCentimeter:
            return f"""{super()._truncate_fraction_digits(self.gram_force_centimeters, fractional_digits)} gf·cm"""
        
        if unit == TorqueUnits.GramForceMeter:
            return f"""{super()._truncate_fraction_digits(self.gram_force_meters, fractional_digits)} gf·m"""
        
        if unit == TorqueUnits.KilogramForceMillimeter:
            return f"""{super()._truncate_fraction_digits(self.kilogram_force_millimeters, fractional_digits)} kgf·mm"""
        
        if unit == TorqueUnits.KilogramForceCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilogram_force_centimeters, fractional_digits)} kgf·cm"""
        
        if unit == TorqueUnits.KilogramForceMeter:
            return f"""{super()._truncate_fraction_digits(self.kilogram_force_meters, fractional_digits)} kgf·m"""
        
        if unit == TorqueUnits.TonneForceMillimeter:
            return f"""{super()._truncate_fraction_digits(self.tonne_force_millimeters, fractional_digits)} tf·mm"""
        
        if unit == TorqueUnits.TonneForceCentimeter:
            return f"""{super()._truncate_fraction_digits(self.tonne_force_centimeters, fractional_digits)} tf·cm"""
        
        if unit == TorqueUnits.TonneForceMeter:
            return f"""{super()._truncate_fraction_digits(self.tonne_force_meters, fractional_digits)} tf·m"""
        
        if unit == TorqueUnits.KilonewtonMillimeter:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_millimeters, fractional_digits)} kN·mm"""
        
        if unit == TorqueUnits.MeganewtonMillimeter:
            return f"""{super()._truncate_fraction_digits(self.meganewton_millimeters, fractional_digits)} MN·mm"""
        
        if unit == TorqueUnits.KilonewtonCentimeter:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_centimeters, fractional_digits)} kN·cm"""
        
        if unit == TorqueUnits.MeganewtonCentimeter:
            return f"""{super()._truncate_fraction_digits(self.meganewton_centimeters, fractional_digits)} MN·cm"""
        
        if unit == TorqueUnits.KilonewtonMeter:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_meters, fractional_digits)} kN·m"""
        
        if unit == TorqueUnits.MeganewtonMeter:
            return f"""{super()._truncate_fraction_digits(self.meganewton_meters, fractional_digits)} MN·m"""
        
        if unit == TorqueUnits.KilopoundForceInch:
            return f"""{super()._truncate_fraction_digits(self.kilopound_force_inches, fractional_digits)} klbf·in"""
        
        if unit == TorqueUnits.MegapoundForceInch:
            return f"""{super()._truncate_fraction_digits(self.megapound_force_inches, fractional_digits)} Mlbf·in"""
        
        if unit == TorqueUnits.KilopoundForceFoot:
            return f"""{super()._truncate_fraction_digits(self.kilopound_force_feet, fractional_digits)} klbf·ft"""
        
        if unit == TorqueUnits.MegapoundForceFoot:
            return f"""{super()._truncate_fraction_digits(self.megapound_force_feet, fractional_digits)} Mlbf·ft"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: TorqueUnits = TorqueUnits.NewtonMeter) -> str:
        """
        Get Torque unit abbreviation.
        Note! the default abbreviation for Torque is NewtonMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TorqueUnits.NewtonMillimeter:
            return """N·mm"""
        
        if unit_abbreviation == TorqueUnits.NewtonCentimeter:
            return """N·cm"""
        
        if unit_abbreviation == TorqueUnits.NewtonMeter:
            return """N·m"""
        
        if unit_abbreviation == TorqueUnits.PoundalFoot:
            return """pdl·ft"""
        
        if unit_abbreviation == TorqueUnits.PoundForceInch:
            return """lbf·in"""
        
        if unit_abbreviation == TorqueUnits.PoundForceFoot:
            return """lbf·ft"""
        
        if unit_abbreviation == TorqueUnits.GramForceMillimeter:
            return """gf·mm"""
        
        if unit_abbreviation == TorqueUnits.GramForceCentimeter:
            return """gf·cm"""
        
        if unit_abbreviation == TorqueUnits.GramForceMeter:
            return """gf·m"""
        
        if unit_abbreviation == TorqueUnits.KilogramForceMillimeter:
            return """kgf·mm"""
        
        if unit_abbreviation == TorqueUnits.KilogramForceCentimeter:
            return """kgf·cm"""
        
        if unit_abbreviation == TorqueUnits.KilogramForceMeter:
            return """kgf·m"""
        
        if unit_abbreviation == TorqueUnits.TonneForceMillimeter:
            return """tf·mm"""
        
        if unit_abbreviation == TorqueUnits.TonneForceCentimeter:
            return """tf·cm"""
        
        if unit_abbreviation == TorqueUnits.TonneForceMeter:
            return """tf·m"""
        
        if unit_abbreviation == TorqueUnits.KilonewtonMillimeter:
            return """kN·mm"""
        
        if unit_abbreviation == TorqueUnits.MeganewtonMillimeter:
            return """MN·mm"""
        
        if unit_abbreviation == TorqueUnits.KilonewtonCentimeter:
            return """kN·cm"""
        
        if unit_abbreviation == TorqueUnits.MeganewtonCentimeter:
            return """MN·cm"""
        
        if unit_abbreviation == TorqueUnits.KilonewtonMeter:
            return """kN·m"""
        
        if unit_abbreviation == TorqueUnits.MeganewtonMeter:
            return """MN·m"""
        
        if unit_abbreviation == TorqueUnits.KilopoundForceInch:
            return """klbf·in"""
        
        if unit_abbreviation == TorqueUnits.MegapoundForceInch:
            return """Mlbf·in"""
        
        if unit_abbreviation == TorqueUnits.KilopoundForceFoot:
            return """klbf·ft"""
        
        if unit_abbreviation == TorqueUnits.MegapoundForceFoot:
            return """Mlbf·ft"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ApparentPowerUnits(Enum):
        """
            ApparentPowerUnits enumeration
        """
        
        Voltampere = 'Voltampere'
        """
            
        """
        
        Microvoltampere = 'Microvoltampere'
        """
            
        """
        
        Millivoltampere = 'Millivoltampere'
        """
            
        """
        
        Kilovoltampere = 'Kilovoltampere'
        """
            
        """
        
        Megavoltampere = 'Megavoltampere'
        """
            
        """
        
        Gigavoltampere = 'Gigavoltampere'
        """
            
        """
        

class ApparentPowerDto:
    """
    A DTO representation of a ApparentPower

    Attributes:
        value (float): The value of the ApparentPower.
        unit (ApparentPowerUnits): The specific unit that the ApparentPower value is representing.
    """

    def __init__(self, value: float, unit: ApparentPowerUnits):
        """
        Create a new DTO representation of a ApparentPower

        Parameters:
            value (float): The value of the ApparentPower.
            unit (ApparentPowerUnits): The specific unit that the ApparentPower value is representing.
        """
        self.value: float = value
        """
        The value of the ApparentPower
        """
        self.unit: ApparentPowerUnits = unit
        """
        The specific unit that the ApparentPower value is representing
        """

    def to_json(self):
        """
        Get a ApparentPower DTO JSON object representing the current unit.

        :return: JSON object represents ApparentPower DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Voltampere"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ApparentPower DTO from a json representation.

        :param data: The ApparentPower DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Voltampere"}
        :return: A new instance of ApparentPowerDto.
        :rtype: ApparentPowerDto
        """
        return ApparentPowerDto(value=data["value"], unit=ApparentPowerUnits(data["unit"]))


class ApparentPower(AbstractMeasure):
    """
    Power engineers measure apparent power as the magnitude of the vector sum of active and reactive power. Apparent power is the product of the root-mean-square of voltage and current.

    Args:
        value (float): The value.
        from_unit (ApparentPowerUnits): The ApparentPower unit to create from, The default unit is Voltampere
    """
    def __init__(self, value: float, from_unit: ApparentPowerUnits = ApparentPowerUnits.Voltampere):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__voltamperes = None
        
        self.__microvoltamperes = None
        
        self.__millivoltamperes = None
        
        self.__kilovoltamperes = None
        
        self.__megavoltamperes = None
        
        self.__gigavoltamperes = None
        

    def convert(self, unit: ApparentPowerUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ApparentPowerUnits = ApparentPowerUnits.Voltampere) -> ApparentPowerDto:
        """
        Get a new instance of ApparentPower DTO representing the current unit.

        :param hold_in_unit: The specific ApparentPower unit to store the ApparentPower value in the DTO representation.
        :type hold_in_unit: ApparentPowerUnits
        :return: A new instance of ApparentPowerDto.
        :rtype: ApparentPowerDto
        """
        return ApparentPowerDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ApparentPowerUnits = ApparentPowerUnits.Voltampere):
        """
        Get a ApparentPower DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ApparentPower unit to store the ApparentPower value in the DTO representation.
        :type hold_in_unit: ApparentPowerUnits
        :return: JSON object represents ApparentPower DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Voltampere"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(apparent_power_dto: ApparentPowerDto):
        """
        Obtain a new instance of ApparentPower from a DTO unit object.

        :param apparent_power_dto: The ApparentPower DTO representation.
        :type apparent_power_dto: ApparentPowerDto
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower(apparent_power_dto.value, apparent_power_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ApparentPower from a DTO unit json representation.

        :param data: The ApparentPower DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Voltampere"}
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower.from_dto(ApparentPowerDto.from_json(data))

    def __convert_from_base(self, from_unit: ApparentPowerUnits) -> float:
        value = self._value
        
        if from_unit == ApparentPowerUnits.Voltampere:
            return (value)
        
        if from_unit == ApparentPowerUnits.Microvoltampere:
            return ((value) / 1e-06)
        
        if from_unit == ApparentPowerUnits.Millivoltampere:
            return ((value) / 0.001)
        
        if from_unit == ApparentPowerUnits.Kilovoltampere:
            return ((value) / 1000.0)
        
        if from_unit == ApparentPowerUnits.Megavoltampere:
            return ((value) / 1000000.0)
        
        if from_unit == ApparentPowerUnits.Gigavoltampere:
            return ((value) / 1000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ApparentPowerUnits) -> float:
        
        if to_unit == ApparentPowerUnits.Voltampere:
            return (value)
        
        if to_unit == ApparentPowerUnits.Microvoltampere:
            return ((value) * 1e-06)
        
        if to_unit == ApparentPowerUnits.Millivoltampere:
            return ((value) * 0.001)
        
        if to_unit == ApparentPowerUnits.Kilovoltampere:
            return ((value) * 1000.0)
        
        if to_unit == ApparentPowerUnits.Megavoltampere:
            return ((value) * 1000000.0)
        
        if to_unit == ApparentPowerUnits.Gigavoltampere:
            return ((value) * 1000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_voltamperes(voltamperes: float):
        """
        Create a new instance of ApparentPower from a value in voltamperes.

        

        :param meters: The ApparentPower value in voltamperes.
        :type voltamperes: float
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower(voltamperes, ApparentPowerUnits.Voltampere)

    
    @staticmethod
    def from_microvoltamperes(microvoltamperes: float):
        """
        Create a new instance of ApparentPower from a value in microvoltamperes.

        

        :param meters: The ApparentPower value in microvoltamperes.
        :type microvoltamperes: float
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower(microvoltamperes, ApparentPowerUnits.Microvoltampere)

    
    @staticmethod
    def from_millivoltamperes(millivoltamperes: float):
        """
        Create a new instance of ApparentPower from a value in millivoltamperes.

        

        :param meters: The ApparentPower value in millivoltamperes.
        :type millivoltamperes: float
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower(millivoltamperes, ApparentPowerUnits.Millivoltampere)

    
    @staticmethod
    def from_kilovoltamperes(kilovoltamperes: float):
        """
        Create a new instance of ApparentPower from a value in kilovoltamperes.

        

        :param meters: The ApparentPower value in kilovoltamperes.
        :type kilovoltamperes: float
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower(kilovoltamperes, ApparentPowerUnits.Kilovoltampere)

    
    @staticmethod
    def from_megavoltamperes(megavoltamperes: float):
        """
        Create a new instance of ApparentPower from a value in megavoltamperes.

        

        :param meters: The ApparentPower value in megavoltamperes.
        :type megavoltamperes: float
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower(megavoltamperes, ApparentPowerUnits.Megavoltampere)

    
    @staticmethod
    def from_gigavoltamperes(gigavoltamperes: float):
        """
        Create a new instance of ApparentPower from a value in gigavoltamperes.

        

        :param meters: The ApparentPower value in gigavoltamperes.
        :type gigavoltamperes: float
        :return: A new instance of ApparentPower.
        :rtype: ApparentPower
        """
        return ApparentPower(gigavoltamperes, ApparentPowerUnits.Gigavoltampere)

    
    @property
    def voltamperes(self) -> float:
        """
        
        """
        if self.__voltamperes != None:
            return self.__voltamperes
        self.__voltamperes = self.__convert_from_base(ApparentPowerUnits.Voltampere)
        return self.__voltamperes

    
    @property
    def microvoltamperes(self) -> float:
        """
        
        """
        if self.__microvoltamperes != None:
            return self.__microvoltamperes
        self.__microvoltamperes = self.__convert_from_base(ApparentPowerUnits.Microvoltampere)
        return self.__microvoltamperes

    
    @property
    def millivoltamperes(self) -> float:
        """
        
        """
        if self.__millivoltamperes != None:
            return self.__millivoltamperes
        self.__millivoltamperes = self.__convert_from_base(ApparentPowerUnits.Millivoltampere)
        return self.__millivoltamperes

    
    @property
    def kilovoltamperes(self) -> float:
        """
        
        """
        if self.__kilovoltamperes != None:
            return self.__kilovoltamperes
        self.__kilovoltamperes = self.__convert_from_base(ApparentPowerUnits.Kilovoltampere)
        return self.__kilovoltamperes

    
    @property
    def megavoltamperes(self) -> float:
        """
        
        """
        if self.__megavoltamperes != None:
            return self.__megavoltamperes
        self.__megavoltamperes = self.__convert_from_base(ApparentPowerUnits.Megavoltampere)
        return self.__megavoltamperes

    
    @property
    def gigavoltamperes(self) -> float:
        """
        
        """
        if self.__gigavoltamperes != None:
            return self.__gigavoltamperes
        self.__gigavoltamperes = self.__convert_from_base(ApparentPowerUnits.Gigavoltampere)
        return self.__gigavoltamperes

    
    def to_string(self, unit: ApparentPowerUnits = ApparentPowerUnits.Voltampere, fractional_digits: int = None) -> str:
        """
        Format the ApparentPower to a string.
        
        Note: the default format for ApparentPower is Voltampere.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ApparentPower. Default is 'Voltampere'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ApparentPowerUnits.Voltampere:
            return f"""{super()._truncate_fraction_digits(self.voltamperes, fractional_digits)} VA"""
        
        if unit == ApparentPowerUnits.Microvoltampere:
            return f"""{super()._truncate_fraction_digits(self.microvoltamperes, fractional_digits)} μVA"""
        
        if unit == ApparentPowerUnits.Millivoltampere:
            return f"""{super()._truncate_fraction_digits(self.millivoltamperes, fractional_digits)} mVA"""
        
        if unit == ApparentPowerUnits.Kilovoltampere:
            return f"""{super()._truncate_fraction_digits(self.kilovoltamperes, fractional_digits)} kVA"""
        
        if unit == ApparentPowerUnits.Megavoltampere:
            return f"""{super()._truncate_fraction_digits(self.megavoltamperes, fractional_digits)} MVA"""
        
        if unit == ApparentPowerUnits.Gigavoltampere:
            return f"""{super()._truncate_fraction_digits(self.gigavoltamperes, fractional_digits)} GVA"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ApparentPowerUnits = ApparentPowerUnits.Voltampere) -> str:
        """
        Get ApparentPower unit abbreviation.
        Note! the default abbreviation for ApparentPower is Voltampere.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ApparentPowerUnits.Voltampere:
            return """VA"""
        
        if unit_abbreviation == ApparentPowerUnits.Microvoltampere:
            return """μVA"""
        
        if unit_abbreviation == ApparentPowerUnits.Millivoltampere:
            return """mVA"""
        
        if unit_abbreviation == ApparentPowerUnits.Kilovoltampere:
            return """kVA"""
        
        if unit_abbreviation == ApparentPowerUnits.Megavoltampere:
            return """MVA"""
        
        if unit_abbreviation == ApparentPowerUnits.Gigavoltampere:
            return """GVA"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class LuminousIntensityUnits(Enum):
        """
            LuminousIntensityUnits enumeration
        """
        
        Candela = 'Candela'
        """
            
        """
        

class LuminousIntensityDto:
    """
    A DTO representation of a LuminousIntensity

    Attributes:
        value (float): The value of the LuminousIntensity.
        unit (LuminousIntensityUnits): The specific unit that the LuminousIntensity value is representing.
    """

    def __init__(self, value: float, unit: LuminousIntensityUnits):
        """
        Create a new DTO representation of a LuminousIntensity

        Parameters:
            value (float): The value of the LuminousIntensity.
            unit (LuminousIntensityUnits): The specific unit that the LuminousIntensity value is representing.
        """
        self.value: float = value
        """
        The value of the LuminousIntensity
        """
        self.unit: LuminousIntensityUnits = unit
        """
        The specific unit that the LuminousIntensity value is representing
        """

    def to_json(self):
        """
        Get a LuminousIntensity DTO JSON object representing the current unit.

        :return: JSON object represents LuminousIntensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Candela"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of LuminousIntensity DTO from a json representation.

        :param data: The LuminousIntensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Candela"}
        :return: A new instance of LuminousIntensityDto.
        :rtype: LuminousIntensityDto
        """
        return LuminousIntensityDto(value=data["value"], unit=LuminousIntensityUnits(data["unit"]))


class LuminousIntensity(AbstractMeasure):
    """
    In photometry, luminous intensity is a measure of the wavelength-weighted power emitted by a light source in a particular direction per unit solid angle, based on the luminosity function, a standardized model of the sensitivity of the human eye.

    Args:
        value (float): The value.
        from_unit (LuminousIntensityUnits): The LuminousIntensity unit to create from, The default unit is Candela
    """
    def __init__(self, value: float, from_unit: LuminousIntensityUnits = LuminousIntensityUnits.Candela):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__candela = None
        

    def convert(self, unit: LuminousIntensityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: LuminousIntensityUnits = LuminousIntensityUnits.Candela) -> LuminousIntensityDto:
        """
        Get a new instance of LuminousIntensity DTO representing the current unit.

        :param hold_in_unit: The specific LuminousIntensity unit to store the LuminousIntensity value in the DTO representation.
        :type hold_in_unit: LuminousIntensityUnits
        :return: A new instance of LuminousIntensityDto.
        :rtype: LuminousIntensityDto
        """
        return LuminousIntensityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: LuminousIntensityUnits = LuminousIntensityUnits.Candela):
        """
        Get a LuminousIntensity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific LuminousIntensity unit to store the LuminousIntensity value in the DTO representation.
        :type hold_in_unit: LuminousIntensityUnits
        :return: JSON object represents LuminousIntensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Candela"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(luminous_intensity_dto: LuminousIntensityDto):
        """
        Obtain a new instance of LuminousIntensity from a DTO unit object.

        :param luminous_intensity_dto: The LuminousIntensity DTO representation.
        :type luminous_intensity_dto: LuminousIntensityDto
        :return: A new instance of LuminousIntensity.
        :rtype: LuminousIntensity
        """
        return LuminousIntensity(luminous_intensity_dto.value, luminous_intensity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of LuminousIntensity from a DTO unit json representation.

        :param data: The LuminousIntensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Candela"}
        :return: A new instance of LuminousIntensity.
        :rtype: LuminousIntensity
        """
        return LuminousIntensity.from_dto(LuminousIntensityDto.from_json(data))

    def __convert_from_base(self, from_unit: LuminousIntensityUnits) -> float:
        value = self._value
        
        if from_unit == LuminousIntensityUnits.Candela:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LuminousIntensityUnits) -> float:
        
        if to_unit == LuminousIntensityUnits.Candela:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_candela(candela: float):
        """
        Create a new instance of LuminousIntensity from a value in candela.

        

        :param meters: The LuminousIntensity value in candela.
        :type candela: float
        :return: A new instance of LuminousIntensity.
        :rtype: LuminousIntensity
        """
        return LuminousIntensity(candela, LuminousIntensityUnits.Candela)

    
    @property
    def candela(self) -> float:
        """
        
        """
        if self.__candela != None:
            return self.__candela
        self.__candela = self.__convert_from_base(LuminousIntensityUnits.Candela)
        return self.__candela

    
    def to_string(self, unit: LuminousIntensityUnits = LuminousIntensityUnits.Candela, fractional_digits: int = None) -> str:
        """
        Format the LuminousIntensity to a string.
        
        Note: the default format for LuminousIntensity is Candela.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the LuminousIntensity. Default is 'Candela'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == LuminousIntensityUnits.Candela:
            return f"""{super()._truncate_fraction_digits(self.candela, fractional_digits)} cd"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: LuminousIntensityUnits = LuminousIntensityUnits.Candela) -> str:
        """
        Get LuminousIntensity unit abbreviation.
        Note! the default abbreviation for LuminousIntensity is Candela.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LuminousIntensityUnits.Candela:
            return """cd"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MagneticFieldUnits(Enum):
        """
            MagneticFieldUnits enumeration
        """
        
        Tesla = 'Tesla'
        """
            
        """
        
        Gauss = 'Gauss'
        """
            
        """
        
        Nanotesla = 'Nanotesla'
        """
            
        """
        
        Microtesla = 'Microtesla'
        """
            
        """
        
        Millitesla = 'Millitesla'
        """
            
        """
        
        Milligauss = 'Milligauss'
        """
            
        """
        

class MagneticFieldDto:
    """
    A DTO representation of a MagneticField

    Attributes:
        value (float): The value of the MagneticField.
        unit (MagneticFieldUnits): The specific unit that the MagneticField value is representing.
    """

    def __init__(self, value: float, unit: MagneticFieldUnits):
        """
        Create a new DTO representation of a MagneticField

        Parameters:
            value (float): The value of the MagneticField.
            unit (MagneticFieldUnits): The specific unit that the MagneticField value is representing.
        """
        self.value: float = value
        """
        The value of the MagneticField
        """
        self.unit: MagneticFieldUnits = unit
        """
        The specific unit that the MagneticField value is representing
        """

    def to_json(self):
        """
        Get a MagneticField DTO JSON object representing the current unit.

        :return: JSON object represents MagneticField DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Tesla"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MagneticField DTO from a json representation.

        :param data: The MagneticField DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Tesla"}
        :return: A new instance of MagneticFieldDto.
        :rtype: MagneticFieldDto
        """
        return MagneticFieldDto(value=data["value"], unit=MagneticFieldUnits(data["unit"]))


class MagneticField(AbstractMeasure):
    """
    A magnetic field is a force field that is created by moving electric charges (electric currents) and magnetic dipoles, and exerts a force on other nearby moving charges and magnetic dipoles.

    Args:
        value (float): The value.
        from_unit (MagneticFieldUnits): The MagneticField unit to create from, The default unit is Tesla
    """
    def __init__(self, value: float, from_unit: MagneticFieldUnits = MagneticFieldUnits.Tesla):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__teslas = None
        
        self.__gausses = None
        
        self.__nanoteslas = None
        
        self.__microteslas = None
        
        self.__milliteslas = None
        
        self.__milligausses = None
        

    def convert(self, unit: MagneticFieldUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MagneticFieldUnits = MagneticFieldUnits.Tesla) -> MagneticFieldDto:
        """
        Get a new instance of MagneticField DTO representing the current unit.

        :param hold_in_unit: The specific MagneticField unit to store the MagneticField value in the DTO representation.
        :type hold_in_unit: MagneticFieldUnits
        :return: A new instance of MagneticFieldDto.
        :rtype: MagneticFieldDto
        """
        return MagneticFieldDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MagneticFieldUnits = MagneticFieldUnits.Tesla):
        """
        Get a MagneticField DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MagneticField unit to store the MagneticField value in the DTO representation.
        :type hold_in_unit: MagneticFieldUnits
        :return: JSON object represents MagneticField DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Tesla"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(magnetic_field_dto: MagneticFieldDto):
        """
        Obtain a new instance of MagneticField from a DTO unit object.

        :param magnetic_field_dto: The MagneticField DTO representation.
        :type magnetic_field_dto: MagneticFieldDto
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(magnetic_field_dto.value, magnetic_field_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MagneticField from a DTO unit json representation.

        :param data: The MagneticField DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Tesla"}
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField.from_dto(MagneticFieldDto.from_json(data))

    def __convert_from_base(self, from_unit: MagneticFieldUnits) -> float:
        value = self._value
        
        if from_unit == MagneticFieldUnits.Tesla:
            return (value)
        
        if from_unit == MagneticFieldUnits.Gauss:
            return (value * 1e4)
        
        if from_unit == MagneticFieldUnits.Nanotesla:
            return ((value) / 1e-09)
        
        if from_unit == MagneticFieldUnits.Microtesla:
            return ((value) / 1e-06)
        
        if from_unit == MagneticFieldUnits.Millitesla:
            return ((value) / 0.001)
        
        if from_unit == MagneticFieldUnits.Milligauss:
            return ((value * 1e4) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MagneticFieldUnits) -> float:
        
        if to_unit == MagneticFieldUnits.Tesla:
            return (value)
        
        if to_unit == MagneticFieldUnits.Gauss:
            return (value / 1e4)
        
        if to_unit == MagneticFieldUnits.Nanotesla:
            return ((value) * 1e-09)
        
        if to_unit == MagneticFieldUnits.Microtesla:
            return ((value) * 1e-06)
        
        if to_unit == MagneticFieldUnits.Millitesla:
            return ((value) * 0.001)
        
        if to_unit == MagneticFieldUnits.Milligauss:
            return ((value / 1e4) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_teslas(teslas: float):
        """
        Create a new instance of MagneticField from a value in teslas.

        

        :param meters: The MagneticField value in teslas.
        :type teslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(teslas, MagneticFieldUnits.Tesla)

    
    @staticmethod
    def from_gausses(gausses: float):
        """
        Create a new instance of MagneticField from a value in gausses.

        

        :param meters: The MagneticField value in gausses.
        :type gausses: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(gausses, MagneticFieldUnits.Gauss)

    
    @staticmethod
    def from_nanoteslas(nanoteslas: float):
        """
        Create a new instance of MagneticField from a value in nanoteslas.

        

        :param meters: The MagneticField value in nanoteslas.
        :type nanoteslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(nanoteslas, MagneticFieldUnits.Nanotesla)

    
    @staticmethod
    def from_microteslas(microteslas: float):
        """
        Create a new instance of MagneticField from a value in microteslas.

        

        :param meters: The MagneticField value in microteslas.
        :type microteslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(microteslas, MagneticFieldUnits.Microtesla)

    
    @staticmethod
    def from_milliteslas(milliteslas: float):
        """
        Create a new instance of MagneticField from a value in milliteslas.

        

        :param meters: The MagneticField value in milliteslas.
        :type milliteslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(milliteslas, MagneticFieldUnits.Millitesla)

    
    @staticmethod
    def from_milligausses(milligausses: float):
        """
        Create a new instance of MagneticField from a value in milligausses.

        

        :param meters: The MagneticField value in milligausses.
        :type milligausses: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(milligausses, MagneticFieldUnits.Milligauss)

    
    @property
    def teslas(self) -> float:
        """
        
        """
        if self.__teslas != None:
            return self.__teslas
        self.__teslas = self.__convert_from_base(MagneticFieldUnits.Tesla)
        return self.__teslas

    
    @property
    def gausses(self) -> float:
        """
        
        """
        if self.__gausses != None:
            return self.__gausses
        self.__gausses = self.__convert_from_base(MagneticFieldUnits.Gauss)
        return self.__gausses

    
    @property
    def nanoteslas(self) -> float:
        """
        
        """
        if self.__nanoteslas != None:
            return self.__nanoteslas
        self.__nanoteslas = self.__convert_from_base(MagneticFieldUnits.Nanotesla)
        return self.__nanoteslas

    
    @property
    def microteslas(self) -> float:
        """
        
        """
        if self.__microteslas != None:
            return self.__microteslas
        self.__microteslas = self.__convert_from_base(MagneticFieldUnits.Microtesla)
        return self.__microteslas

    
    @property
    def milliteslas(self) -> float:
        """
        
        """
        if self.__milliteslas != None:
            return self.__milliteslas
        self.__milliteslas = self.__convert_from_base(MagneticFieldUnits.Millitesla)
        return self.__milliteslas

    
    @property
    def milligausses(self) -> float:
        """
        
        """
        if self.__milligausses != None:
            return self.__milligausses
        self.__milligausses = self.__convert_from_base(MagneticFieldUnits.Milligauss)
        return self.__milligausses

    
    def to_string(self, unit: MagneticFieldUnits = MagneticFieldUnits.Tesla, fractional_digits: int = None) -> str:
        """
        Format the MagneticField to a string.
        
        Note: the default format for MagneticField is Tesla.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MagneticField. Default is 'Tesla'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MagneticFieldUnits.Tesla:
            return f"""{super()._truncate_fraction_digits(self.teslas, fractional_digits)} T"""
        
        if unit == MagneticFieldUnits.Gauss:
            return f"""{super()._truncate_fraction_digits(self.gausses, fractional_digits)} G"""
        
        if unit == MagneticFieldUnits.Nanotesla:
            return f"""{super()._truncate_fraction_digits(self.nanoteslas, fractional_digits)} nT"""
        
        if unit == MagneticFieldUnits.Microtesla:
            return f"""{super()._truncate_fraction_digits(self.microteslas, fractional_digits)} μT"""
        
        if unit == MagneticFieldUnits.Millitesla:
            return f"""{super()._truncate_fraction_digits(self.milliteslas, fractional_digits)} mT"""
        
        if unit == MagneticFieldUnits.Milligauss:
            return f"""{super()._truncate_fraction_digits(self.milligausses, fractional_digits)} mG"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MagneticFieldUnits = MagneticFieldUnits.Tesla) -> str:
        """
        Get MagneticField unit abbreviation.
        Note! the default abbreviation for MagneticField is Tesla.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MagneticFieldUnits.Tesla:
            return """T"""
        
        if unit_abbreviation == MagneticFieldUnits.Gauss:
            return """G"""
        
        if unit_abbreviation == MagneticFieldUnits.Nanotesla:
            return """nT"""
        
        if unit_abbreviation == MagneticFieldUnits.Microtesla:
            return """μT"""
        
        if unit_abbreviation == MagneticFieldUnits.Millitesla:
            return """mT"""
        
        if unit_abbreviation == MagneticFieldUnits.Milligauss:
            return """mG"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class BrakeSpecificFuelConsumptionUnits(Enum):
        """
            BrakeSpecificFuelConsumptionUnits enumeration
        """
        
        GramPerKiloWattHour = 'GramPerKiloWattHour'
        """
            
        """
        
        KilogramPerJoule = 'KilogramPerJoule'
        """
            
        """
        
        PoundPerMechanicalHorsepowerHour = 'PoundPerMechanicalHorsepowerHour'
        """
            The pound per horse power hour uses mechanical horse power and the imperial pound
        """
        

class BrakeSpecificFuelConsumptionDto:
    """
    A DTO representation of a BrakeSpecificFuelConsumption

    Attributes:
        value (float): The value of the BrakeSpecificFuelConsumption.
        unit (BrakeSpecificFuelConsumptionUnits): The specific unit that the BrakeSpecificFuelConsumption value is representing.
    """

    def __init__(self, value: float, unit: BrakeSpecificFuelConsumptionUnits):
        """
        Create a new DTO representation of a BrakeSpecificFuelConsumption

        Parameters:
            value (float): The value of the BrakeSpecificFuelConsumption.
            unit (BrakeSpecificFuelConsumptionUnits): The specific unit that the BrakeSpecificFuelConsumption value is representing.
        """
        self.value: float = value
        """
        The value of the BrakeSpecificFuelConsumption
        """
        self.unit: BrakeSpecificFuelConsumptionUnits = unit
        """
        The specific unit that the BrakeSpecificFuelConsumption value is representing
        """

    def to_json(self):
        """
        Get a BrakeSpecificFuelConsumption DTO JSON object representing the current unit.

        :return: JSON object represents BrakeSpecificFuelConsumption DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KilogramPerJoule"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of BrakeSpecificFuelConsumption DTO from a json representation.

        :param data: The BrakeSpecificFuelConsumption DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KilogramPerJoule"}
        :return: A new instance of BrakeSpecificFuelConsumptionDto.
        :rtype: BrakeSpecificFuelConsumptionDto
        """
        return BrakeSpecificFuelConsumptionDto(value=data["value"], unit=BrakeSpecificFuelConsumptionUnits(data["unit"]))


class BrakeSpecificFuelConsumption(AbstractMeasure):
    """
    Brake specific fuel consumption (BSFC) is a measure of the fuel efficiency of any prime mover that burns fuel and produces rotational, or shaft, power. It is typically used for comparing the efficiency of internal combustion engines with a shaft output.

    Args:
        value (float): The value.
        from_unit (BrakeSpecificFuelConsumptionUnits): The BrakeSpecificFuelConsumption unit to create from, The default unit is KilogramPerJoule
    """
    def __init__(self, value: float, from_unit: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__grams_per_kilo_watt_hour = None
        
        self.__kilograms_per_joule = None
        
        self.__pounds_per_mechanical_horsepower_hour = None
        

    def convert(self, unit: BrakeSpecificFuelConsumptionUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule) -> BrakeSpecificFuelConsumptionDto:
        """
        Get a new instance of BrakeSpecificFuelConsumption DTO representing the current unit.

        :param hold_in_unit: The specific BrakeSpecificFuelConsumption unit to store the BrakeSpecificFuelConsumption value in the DTO representation.
        :type hold_in_unit: BrakeSpecificFuelConsumptionUnits
        :return: A new instance of BrakeSpecificFuelConsumptionDto.
        :rtype: BrakeSpecificFuelConsumptionDto
        """
        return BrakeSpecificFuelConsumptionDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule):
        """
        Get a BrakeSpecificFuelConsumption DTO JSON object representing the current unit.

        :param hold_in_unit: The specific BrakeSpecificFuelConsumption unit to store the BrakeSpecificFuelConsumption value in the DTO representation.
        :type hold_in_unit: BrakeSpecificFuelConsumptionUnits
        :return: JSON object represents BrakeSpecificFuelConsumption DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KilogramPerJoule"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(brake_specific_fuel_consumption_dto: BrakeSpecificFuelConsumptionDto):
        """
        Obtain a new instance of BrakeSpecificFuelConsumption from a DTO unit object.

        :param brake_specific_fuel_consumption_dto: The BrakeSpecificFuelConsumption DTO representation.
        :type brake_specific_fuel_consumption_dto: BrakeSpecificFuelConsumptionDto
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption(brake_specific_fuel_consumption_dto.value, brake_specific_fuel_consumption_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of BrakeSpecificFuelConsumption from a DTO unit json representation.

        :param data: The BrakeSpecificFuelConsumption DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KilogramPerJoule"}
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption.from_dto(BrakeSpecificFuelConsumptionDto.from_json(data))

    def __convert_from_base(self, from_unit: BrakeSpecificFuelConsumptionUnits) -> float:
        value = self._value
        
        if from_unit == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return (value * 3.6e9)
        
        if from_unit == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return (value)
        
        if from_unit == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return (value / 1.689659410672e-7)
        
        return None


    def __convert_to_base(self, value: float, to_unit: BrakeSpecificFuelConsumptionUnits) -> float:
        
        if to_unit == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return (value / 3.6e9)
        
        if to_unit == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return (value)
        
        if to_unit == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return (value * 1.689659410672e-7)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_grams_per_kilo_watt_hour(grams_per_kilo_watt_hour: float):
        """
        Create a new instance of BrakeSpecificFuelConsumption from a value in grams_per_kilo_watt_hour.

        

        :param meters: The BrakeSpecificFuelConsumption value in grams_per_kilo_watt_hour.
        :type grams_per_kilo_watt_hour: float
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption(grams_per_kilo_watt_hour, BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour)

    
    @staticmethod
    def from_kilograms_per_joule(kilograms_per_joule: float):
        """
        Create a new instance of BrakeSpecificFuelConsumption from a value in kilograms_per_joule.

        

        :param meters: The BrakeSpecificFuelConsumption value in kilograms_per_joule.
        :type kilograms_per_joule: float
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption(kilograms_per_joule, BrakeSpecificFuelConsumptionUnits.KilogramPerJoule)

    
    @staticmethod
    def from_pounds_per_mechanical_horsepower_hour(pounds_per_mechanical_horsepower_hour: float):
        """
        Create a new instance of BrakeSpecificFuelConsumption from a value in pounds_per_mechanical_horsepower_hour.

        The pound per horse power hour uses mechanical horse power and the imperial pound

        :param meters: The BrakeSpecificFuelConsumption value in pounds_per_mechanical_horsepower_hour.
        :type pounds_per_mechanical_horsepower_hour: float
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption(pounds_per_mechanical_horsepower_hour, BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour)

    
    @property
    def grams_per_kilo_watt_hour(self) -> float:
        """
        
        """
        if self.__grams_per_kilo_watt_hour != None:
            return self.__grams_per_kilo_watt_hour
        self.__grams_per_kilo_watt_hour = self.__convert_from_base(BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour)
        return self.__grams_per_kilo_watt_hour

    
    @property
    def kilograms_per_joule(self) -> float:
        """
        
        """
        if self.__kilograms_per_joule != None:
            return self.__kilograms_per_joule
        self.__kilograms_per_joule = self.__convert_from_base(BrakeSpecificFuelConsumptionUnits.KilogramPerJoule)
        return self.__kilograms_per_joule

    
    @property
    def pounds_per_mechanical_horsepower_hour(self) -> float:
        """
        The pound per horse power hour uses mechanical horse power and the imperial pound
        """
        if self.__pounds_per_mechanical_horsepower_hour != None:
            return self.__pounds_per_mechanical_horsepower_hour
        self.__pounds_per_mechanical_horsepower_hour = self.__convert_from_base(BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour)
        return self.__pounds_per_mechanical_horsepower_hour

    
    def to_string(self, unit: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule, fractional_digits: int = None) -> str:
        """
        Format the BrakeSpecificFuelConsumption to a string.
        
        Note: the default format for BrakeSpecificFuelConsumption is KilogramPerJoule.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the BrakeSpecificFuelConsumption. Default is 'KilogramPerJoule'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return f"""{super()._truncate_fraction_digits(self.grams_per_kilo_watt_hour, fractional_digits)} g/kWh"""
        
        if unit == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_joule, fractional_digits)} kg/J"""
        
        if unit == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return f"""{super()._truncate_fraction_digits(self.pounds_per_mechanical_horsepower_hour, fractional_digits)} lb/hph"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule) -> str:
        """
        Get BrakeSpecificFuelConsumption unit abbreviation.
        Note! the default abbreviation for BrakeSpecificFuelConsumption is KilogramPerJoule.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return """g/kWh"""
        
        if unit_abbreviation == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return """kg/J"""
        
        if unit_abbreviation == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return """lb/hph"""
        